/*
 * Created on 31.01.2007
 *
 */
package common.core;

import org.apache.log4j.Logger;

/**
 * <pre>
 *                 Copyright (c) 2007 Dominik Schulz
 *                 Copyright (c) 2007 Florian Lindner
 *                 Copyright (c) 2007 Philip Hartmann
 *
 *                 This file is part of jUPNP.
 *
 *                 jUPNP is free software; you can redistribute it and/or modify
 *                 it under the terms of the GNU General Public License as published by
 *                 the Free Software Foundation; either version 2 of the License, or
 *                 (at your option) any later version.
 *
 *                 jUPNP is distributed in the hope that it will be useful,
 *                 but WITHOUT ANY WARRANTY; without even the implied warranty of
 *                 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *                 GNU General Public License for more details.
 *
 *                 You should have received a copy of the GNU General Public License
 *                 along with jUPNP; if not, write to the Free Software
 *                 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 * This thread does little more than printing a "--- MARK ---" to the console/log every 10 seconds.
 *
 * Additionally it cleans up the expired subscriptions.
 *
 * "There is no theory of evolution. Just a list of animals Chuck Norris allows to live."
 */
public class HeartbeatThread implements Runnable {

	static Logger	logger	= Logger.getLogger(HeartbeatThread.class);

	private Config	config	= null;

	private int		rounds	= 0;

	public HeartbeatThread(Config config) {
		this.config = config;
	}

	/**
	 * Runs the heartbeatThread, does a subscriptions-cleanup
	 */
	public void run() {
		logger.trace("Starting Hearbeat ...");
		while (true) {
			if (Config.SILENT) {
				logger.trace("--- MARK ---");
			} else {
				logger.info("--- MARK ---");
			}
			// remove expired subscriptions
			if (this.config.getDevice() != null) {
				this.config.getDevice().subscriptionsCleanUp();
			}
			// send an ssdp alive every 5 minutes (5 min * 6 * 10 sec)
			if (this.rounds >= 5 * 6) {
				this.rounds = 0;
				this.config.getDevice().sendSSDPAlive();
			} else {
				this.rounds++;
			}
			try {
				// wait 10 seconds
				Thread.sleep(10 * 1000);
			} catch (InterruptedException e) {
				logger.trace(e);
			}
		}
	}

}
