/*
 * Created on 11.01.2007
 *
 */
package common.http;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.net.URL;

import org.apache.log4j.Logger;

/**
 * <pre>
 *               Copyright (c) 2007 Dominik Schulz
 *               Copyright (c) 2007 Florian Lindner
 *               Copyright (c) 2007 Philip Hartmann
 *
 *               This file is part of jUPNP.
 *
 *               jUPNP is free software; you can redistribute it and/or modify
 *               it under the terms of the GNU General Public License as published by
 *               the Free Software Foundation; either version 2 of the License, or
 *               (at your option) any later version.
 *
 *               jUPNP is distributed in the hope that it will be useful,
 *               but WITHOUT ANY WARRANTY; without even the implied warranty of
 *               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *               GNU General Public License for more details.
 *
 *               You should have received a copy of the GNU General Public License
 *               along with jUPNP; if not, write to the Free Software
 *               Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * This class is used to issue a Http Request, similar to using the URL.getInputStream() facility with the diffence that
 * this way we are able to transmit a body when requesting the url which is used for ex. when making a SOAP-Request.
 *
 * @author tex
 *
 * "There is no chin under Chuck Norris' Beard. There is only another fist."
 */
public class CustomHttpRequest {

	static Logger			logger			= Logger.getLogger(CustomHttpRequest.class);

	private URL				targetUrl		= null;

	private InetAddress		targetHost		= null;

	private int				targetPort		= 0;

	private Socket			clientSocket	= null;

	private Header			header			= null;

	private String			httpMethod		= "GET";

	private String			payload			= "";

	private OutputStream	out				= null;

	private BufferedReader	br				= null;

	private String			response		= "";

	private String			responseHeader	= "";

	/**
	 * A custom request for any kind of HTTP-requests. For example in Java there was no construct to do a HTTP requests
	 * different than GET (e.g. POST, etc.).
	 */
	public CustomHttpRequest(URL target, String httpMethod) throws IOException {
		this.targetUrl = target;
		this.httpMethod = httpMethod;
		this.targetHost = InetAddress.getByName(this.targetUrl.getHost());
		this.targetPort = this.targetUrl.getPort();
		this.clientSocket = new Socket(this.targetHost, this.targetPort);
		this.header = new Header(clientSocket.getOutputStream());
		this.out = this.clientSocket.getOutputStream();
		this.br = new BufferedReader(new InputStreamReader(this.clientSocket.getInputStream()));
		this.header.addHeaderLine(this.httpMethod + " " + this.targetUrl.getPath() + " HTTP/"
				+ ConfigHttpd.HTTP_VERSION);
		this.header.addHeaderLine("HOST: " + this.targetHost.getHostAddress().toString() + ":" + this.targetPort);
	}

	/**
	 * Transforms the request into a String
	 */
	public String toString() {
		return "TargetURL: " + this.targetUrl + ", TargetHost: " + this.targetHost + ", TargetPort: " + this.targetPort
				+ ", Header: " + this.header.toString() + ", HttpMethod: " + this.httpMethod + ", Payload: "
				+ this.payload + ", Response: " + this.response + ", ResponseHeader: " + this.responseHeader;
	}

	/**
	 * Adds a header line. The Line-Feed will be added later.
	 *
	 * @param line
	 */
	public void addHeaderLine(String line) {
		this.header.addHeaderLine(line);
	}

	/**
	 * Set the payload, i.e. the body of this request.
	 *
	 * @param pl
	 */
	public void setPayload(String pl) {
		this.payload = pl;
	}

	/**
	 * returns the response to this request.
	 *
	 * @return
	 */
	public String getResponse() {
		return this.response;
	}

	/**
	 * Sends the HTTP-Request containing the payload, contentlength, etc.
	 *
	 * @throws IOException
	 */
	public void sendRequest() throws IOException {
		//create and send the header
		String finalizedPayload = this.payload + ConfigHttpd.CRLF;
		header.addHeaderLine("Content-Length: " + finalizedPayload.length());
		logger.trace("Header:\n" + header.toString());
		logger.trace("Payload:\n" + payload);
		header.sendHeader();
		this.out.write(finalizedPayload.getBytes());
		String line = "";
		StringBuilder sb = new StringBuilder();
		try {
			while ((line = br.readLine()).length() != 0) {
				sb.append(line + "\n");
			}
		} catch (NullPointerException e) {
			logger.trace("Got NullPointerException");
		}
		this.responseHeader = sb.toString();
		sb = new StringBuilder();
		while ((line = br.readLine()) != null) {
			sb.append(line + "\n");
		}
		this.response = sb.toString();
	}

	public String getResponseHeader() {
		return this.responseHeader;
	}
}
