/*
 * Created on 25.10.2006
 */
package common.http;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

import org.apache.log4j.Logger;

import common.core.Config;
import controlpoint.core.ControlPoint;
import controlpoint.http.HttpRequestCP;
import device.http.HttpRequestDev;

/**
 * <pre>
 *           Copyright (c) 2007 Dominik Schulz
 *           Copyright (c) 2007 Florian Lindner
 *           Copyright (c) 2007 Philip Hartmann
 *
 *           This file is part of jHTTPd.
 *
 *           jHTTPd is free software; you can redistribute it and/or modify
 *           it under the terms of the GNU General Public License as published by
 *           the Free Software Foundation; either version 2 of the License, or
 *           (at your option) any later version.
 *
 *           jHTTPd is distributed in the hope that it will be useful,
 *           but WITHOUT ANY WARRANTY; without even the implied warranty of
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *           GNU General Public License for more details.
 *
 *           You should have received a copy of the GNU General Public License
 *           along with jHTTPd; if not, write to the Free Software
 *           Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author Dominik
 * @author Philip
 *
 * "The leading causes of death in the United States are: 1. Heart Disease 2. Chuck Norris 3. Cancer"
 *
 */
public class WebServer implements Runnable {

	static Logger			logger		= Logger.getLogger(WebServer.class);

	private Config			config		= null;

	private ConfigHttpd		configHttpd	= null;

	private ControlPoint	cp			= null;

	private boolean			forCp		= false;

	/**
	 *
	 * @param conf
	 * @param configHttp
	 */
	private WebServer(Config conf, ConfigHttpd configHttp) {
		this.config = conf;
		this.configHttpd = configHttp;
	}

	/**
	 *
	 * @param conf
	 * @param configHttpd
	 * @param cp
	 * @return
	 */
	public static WebServer getWebServerForCP(Config conf, ConfigHttpd configHttpd, ControlPoint cp) {
		WebServer ws = new WebServer(conf, configHttpd);
		ws.cp = cp;
		ws.forCp = true;
		return ws;
	}

	/**
	 *
	 * @param conf
	 * @param configHttpd
	 * @return
	 */
	public static WebServer getWebServerForDev(Config conf, ConfigHttpd configHttpd) {
		WebServer ws = new WebServer(conf, configHttpd);
		ws.forCp = false;
		return ws;
	}

	public void run() {
		ServerSocket listenSocket = this.config.getHttpServerSocket();
		logger.trace("run() - Starting WebServer (TCP) on Port " + listenSocket.getLocalPort());
		// Process HTTP service requests in an infinite loop
		for (;;) {
			// Listen for a TCP connection request
			Socket connectionSocket = null;
			try {
				connectionSocket = listenSocket.accept();
			} catch (IOException e) {
				logger.trace("IOException occurred ", e);
			}

			Thread thread = null;
			// Construct an object to process the HTTP request message
			if (this.forCp) {
				HttpRequestCP request = null;
				try {
					request = new HttpRequestCP(connectionSocket, configHttpd, this.config, this.cp);
					// Create a new thread to process the request
					thread = new Thread(request);
				} catch (Exception e) {
					logger.trace("Exception occurred ", e);
				}
			} else {
				HttpRequestDev request = null;
				try {
					request = new HttpRequestDev(connectionSocket, configHttpd, this.config);
					// Create a new thread to process the request
					thread = new Thread(request);
				} catch (Exception e) {
					logger.trace("Exception occurred ", e);
				}

			}
			// Start the thread
			thread.start();
		}
	}
}
