/*
 * Created on 25.10.2006
 */
package common.http;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;

import org.apache.log4j.Logger;

import common.core.Config;
import controlpoint.core.ControlPoint;
import controlpoint.http.HttpRequestCP;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 * This class implements a webserver for the UDP protocol as required by UPnP/HTTPU.
 *
 * "Chuck Norris drives an ice cream truck covered in human skulls."
 *
 */
public class WebServerUDP implements Runnable {

	static Logger			logger		= Logger.getLogger(WebServerUDP.class);

	private Config			config		= null;

	private ConfigHttpd		configHttpd	= null;

	private ControlPoint	cp			= null;

	@SuppressWarnings("unused")
	private boolean			forCp		= false;

	private WebServerUDP() {

	}

	/**
	 * A static factory method instead of a constructor.
	 *
	 * @param conf
	 * @param configHttp
	 * @param cp
	 * @return
	 */
	public static WebServerUDP getWebServerUdpForCP(Config conf, ConfigHttpd configHttp, ControlPoint cp) {
		WebServerUDP wsu = new WebServerUDP();
		wsu.config = conf;
		wsu.configHttpd = configHttp;
		wsu.cp = cp;
		wsu.forCp = true;
		return wsu;
	}

	/**
	 * A static factory method instead of a constructor.
	 *
	 * @param conf
	 * @param configHttp
	 * @return
	 */
	public static WebServerUDP getWebServerUdpForDev(Config conf, ConfigHttpd configHttp) {
		WebServerUDP wsu = new WebServerUDP();
		wsu.config = conf;
		wsu.configHttpd = configHttp;
		wsu.forCp = false;
		return wsu;
	}

	/**
	 * Runs the HTTP Webserver
	 */
	public void run() {
		configHttpd.setPort(this.config.getHttpServerPort());
		// uses a Datagramsocket for the UDP packets
		DatagramSocket ds = config.getHttpUdpServerSocket();
		logger.trace("run() - Starting WebServer (UDP) on Port " + ds.getLocalPort());
		// Process HTTP service requests in an infinite loop
		for (;;) {
			byte[] buffer = new byte[1000];
			byte[] outBuff = new byte[1000];
			DatagramPacket dp = new DatagramPacket(buffer, buffer.length);
			// Listen for a UDP connection request
			try {
				ds.receive(dp);
			} catch (IOException e) {
				logger.trace("IOException Occurred ", e);
			}

			// Construct an object to process the HTTP request message
			HttpRequestCP request = null;
			try {
				request = new HttpRequestCP(dp.getData(), outBuff, configHttpd, this.config, this.cp);
			} catch (Exception e) {
				logger.trace("Exception occurred ", e);
			}
			// Create a new thread to process the request
			Thread thread = new Thread(request);
			// Start the thread
			thread.start();

			while (thread.isAlive()) {
				try {
					// refresh every 250 ms
					Thread.sleep(250);
				} catch (InterruptedException e) {
					logger.trace(e);
				}
			}
			// send Datargam reply
			DatagramPacket dpReply = new DatagramPacket(outBuff, outBuff.length);
			dpReply.setAddress(dp.getAddress());
			dpReply.setPort(dp.getPort());
			try {
				ds.send(dpReply);
			} catch (IOException e) {
				logger.trace(e);
			}
		}
	}
}
