package common.multicast;

import java.io.IOException;
import java.net.DatagramPacket;
import java.util.Arrays;

import org.apache.log4j.Logger;

import common.core.Config;
import common.http.ConfigHttpd;
import controlpoint.core.ControlPoint;
import controlpoint.http.HttpRequestCP;
import device.http.HttpRequestDev;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 * This class implements a "webserver" listening on the UDP-Multicast channel, as defined in the config, which is
 * required by UPnP (HTTPMU).
 *
 * "Chuck Norris doesn't go hunting.... CHUCK NORRIS GOES KILLING"
 *
 */
public class MulticastListenThread implements Runnable {

	static Logger				logger		= Logger.getLogger(MulticastListenThread.class);

	private Config				conf		= null;

	private ConfigHttpd			configHttpd	= null;

	private ControlPoint		cp			= null;

	private boolean				forCp		= false;

	private static final int	BUFFER_SIZE	= 64 * 1024;										// A buffer of 64KB

	// should be big enough

	/**
	 * A multicastlistenthread ist resonsable to receive notifications from the devices. These notifications are
	 * multicast messages
	 *
	 * This initializes this as a Device Thread.
	 *
	 * @param conf
	 * @param configHttpd
	 */
	public MulticastListenThread(Config conf, ConfigHttpd configHttpd) {
		this.conf = conf;
		this.configHttpd = configHttpd;
		this.forCp = false;
	}

	/**
	 * A multicastlistenthread ist resonsable to receive notifications from the devices. These notifications are
	 * multicast messages
	 *
	 * This initializes this as a Control-Point Thread.
	 *
	 * @param conf
	 * @param configHttpd
	 * @param cp
	 */
	public MulticastListenThread(Config conf, ConfigHttpd configHttpd, ControlPoint cp) {
		this.conf = conf;
		this.configHttpd = configHttpd;
		this.cp = cp;
		this.forCp = true;
	}

	/**
	 * Runs MultiCastListenThread an expects MultiCast messages
	 */
	public void run() {
		while (!Thread.interrupted()) {
			byte[] inBuff = new byte[BUFFER_SIZE];
			DatagramPacket recv = new DatagramPacket(inBuff, inBuff.length);
			byte[] outBuff = new byte[BUFFER_SIZE];
			DatagramPacket send = null;
			try {
				// If the Datagrampacket is for the ControlPoint
				if (this.forCp) {
					HttpRequestCP request = null;
					this.conf.getMcSocket().receive(recv);
					// we have to check if we actually recieved any data at all!
					if (!Arrays.equals(recv.getData(), new byte[1000])) {
						logger.trace(Thread.currentThread().getName() + " - Recieved: " + new String(recv.getData())
								+ ", ");
						// create an new request of the ControlPoint
						request = new HttpRequestCP(recv.getData(), outBuff, this.configHttpd, this.conf, this.cp, recv
								.getAddress(), recv.getPort());
						Thread requestThread = new Thread(request);
						requestThread.start();
						Thread.sleep(10);
					} else {
						Thread.sleep(50);
					}
					Thread.sleep(10);
					outBuff = request.getOutput();
				} else {
					// if it belongs to a device
					HttpRequestDev request = null;
					this.conf.getMcSocket().receive(recv);
					// we have to check if we actually recieved any data at all!
					if (!Arrays.equals(recv.getData(), new byte[BUFFER_SIZE])) {
						logger.trace(Thread.currentThread().getName() + " - Recieved: " + new String(recv.getData())
								+ ", ");
						request = new HttpRequestDev(recv.getData(), outBuff, this.configHttpd, this.conf, recv
								.getAddress(), recv.getPort());
						Thread requestThread = new Thread(request);
						requestThread.start();
						Thread.sleep(10);
					} else {
						logger.trace("Byte Arrays are equal -> no data recieved!");
						Thread.sleep(50);
					}
					Thread.sleep(10);
					outBuff = request.getOutput();
				}
				logger.trace("MLT: " + Arrays.toString(outBuff));
				// if buffer is not empty
				if (!Arrays.equals(outBuff, new byte[BUFFER_SIZE])) {
					// send Datagrampacket
					send = new DatagramPacket(outBuff, outBuff.length);
					logger.trace("MulticastListenThread: OutBuffer -> " + Arrays.toString(outBuff) + "| Length -> "
							+ outBuff.length);
					send.setAddress(recv.getAddress());
					send.setPort(recv.getPort());
					logger.trace("Port: " + send.getPort());
					this.conf.getMcSocket().send(send);
				}
			} catch (IOException e) {
				logger.trace(e);
			} catch (InterruptedException e) {
				logger.trace(e);
			}
		}

	}

	public ControlPoint getCp() {
		return cp;
	}
}
