package common.structures;

import java.util.ArrayList;

import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 * "When the Boogeyman goes to sleep every night, he checks his closet for Chuck Norris."
 *
 */
public class Action {

	private String				name		= "";

	/**
	 * ArrayList containing the arguments of an action from a device
	 */
	private ArrayList<Argument>	arguments	= new ArrayList<Argument>();

	/**
	 *
	 * @return a XML-Structure (Element) representing this node
	 */
	public Element getXMLElements() {
		Element root = new Element("action");

		Element name = new Element("name");
		name.setText(this.name);
		root.addContent(name);

		Element arguments = new Element("argumentList");
		for (int i = 0, n = this.arguments.size(); i < n; i++) {
			arguments.addContent(this.arguments.get(i).getXMLElements());
		}
		root.addContent(arguments);

		return root;
	}

	/**
	 *
	 * @return a pretty formated XML String representing this node
	 */
	public String getXMLString() {
		// Create an XMLOutPutter to transform the XML-Code
		// print out the XML
		XMLOutputter xmlOut = new XMLOutputter();
		xmlOut.setFormat(Format.getPrettyFormat());
		return xmlOut.outputString(this.getXMLElements());
	}

	/**
	 * @return the arguments
	 */
	public ArrayList<Argument> getArguments() {
		return arguments;
	}

	/**
	 * @param arguments
	 *            the arguments to set
	 */
	public void setArguments(ArrayList<Argument> arguments) {
		this.arguments = arguments;
	}

	/**
	 * add an argument to the arguments-arraylist
	 * @param arg
	 * @return
	 */
	public boolean addArgument(Argument arg) {
		return this.arguments.add(arg);
	}

	public int getArgumentCount() {
		return this.arguments.size();
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *            the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}
}
