package common.structures;

import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 * "There is no theory of evolution. Just a list of creatures Chuck Norris has allowed to live."
 */
public class Argument {

	private boolean			returnValue				= false;

	private String			name					= "";

	private String			direction				= "";

	private String			value					= "";

	private StateVariable	relatedStateVariable	= null;

	private String			rstv					= null;

	/**
	 * @return the direction
	 */
	public String getDirection() {
		return direction;
	}

	/**
	 * @param direction
	 *            the direction to set
	 */
	public void setDirection(String direction) {
		this.direction = direction;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *            the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the relatedStateVariable
	 */
	public StateVariable getRelatedStateVariable() {
		return relatedStateVariable;
	}

	/**
	 * @param relatedStateVariable
	 *            the relatedStateVariable to set
	 */
	public void setRelatedStateVariable(StateVariable relatedStateVariable) {
		this.relatedStateVariable = relatedStateVariable;
	}

	/**
	 * @return the returnValue
	 */
	public boolean isReturnValue() {
		return returnValue;
	}

	/**
	 * @param returnValue
	 *            the returnValue to set
	 */
	public void setReturnValue(boolean returnValue) {
		this.returnValue = returnValue;
	}

	/**
	 * @return the value
	 */
	public String getValue() {
		return value;
	}

	/**
	 * @param value
	 *            the value to set
	 */
	public void setValue(String value) {
		this.value = value;
	}

	/**
	 *
	 * @return a XML-Structure (Element) representing this node
	 */
	public Element getXMLElements() {
		Element root = new Element("argument");

		Element returnValue = new Element("returnValue");
		returnValue.setText(new Boolean(this.returnValue).toString());
		// root.addContent(returnValue);

		Element name = new Element("name");
		name.setText(this.name);
		root.addContent(name);

		Element direction = new Element("direction");
		direction.setText(this.direction);
		root.addContent(direction);

		Element value = new Element("value");
		value.setText(this.value);

		Element relatedStateVariable = new Element("relatedStateVariable");
		if (this.relatedStateVariable != null) {
			relatedStateVariable.setText(this.relatedStateVariable.getName());
		}
		root.addContent(relatedStateVariable);

		return root;
	}

	/**
	 *
	 * @return a pretty formated XML String representing this node
	 */
	public String getXMLString() {
		XMLOutputter xmlOut = new XMLOutputter();
		xmlOut.setFormat(Format.getPrettyFormat());
		return xmlOut.outputString(this.getXMLElements());
	}

	public String getRstv() {
		return rstv;
	}

	public void setRstv(String rstv) {
		this.rstv = rstv;
	}
}
