package common.structures;

import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 * "Outer space exists because it's afraid to be on the same planet with Chuck Norris."
 */
public class Icon {

	private String	mimeType	= "";

	private int		width		= 0;

	private int		height		= 0;

	private int		depth		= 0;

	private String	url			= null;

	public Icon() {
	}

	/**
	 *
	 * @return a XML-Structure (Element) representing this node
	 */
	public Element getXMLElements() {
		Element root = new Element("icon");

		Element mimeType = new Element("mimeType");
		mimeType.setText(this.mimeType);
		root.addContent(mimeType);

		Element width = new Element("width");
		width.setText((new Integer(this.width)).toString());
		root.addContent(width);

		Element height = new Element("height");
		height.setText((new Integer(this.height)).toString());
		root.addContent(height);

		Element depth = new Element("depth");
		depth.setText((new Integer(this.depth)).toString());
		root.addContent(depth);

		Element url = new Element("URL");
		if (this.url != null)
			url.setText(this.url.toString());
		root.addContent(url);

		return root;
	}

	/**
	 *
	 * @return a pretty formated XML String representing this node
	 */
	public String getXMLString() {
		XMLOutputter xmlOut = new XMLOutputter();
		xmlOut.setFormat(Format.getPrettyFormat());
		return xmlOut.outputString(this.getXMLElements());
	}

	/**
	 * @return the depth
	 */
	public int getDepth() {
		return depth;
	}

	/**
	 * @param depth
	 *            the depth to set
	 */
	public void setDepth(int depth) {
		this.depth = depth;
	}

	/**
	 * @return the height
	 */
	public int getHeight() {
		return height;
	}

	/**
	 * @param height
	 *            the height to set
	 */
	public void setHeight(int height) {
		this.height = height;
	}

	/**
	 * @return the mimeType
	 */
	public String getMimeType() {
		return mimeType;
	}

	/**
	 * @param mimeType
	 *            the mimeType to set
	 */
	public void setMimeType(String mimeType) {
		this.mimeType = mimeType;
	}

	/**
	 * @return the url
	 */
	public String getUrl() {
		return url;
	}

	/**
	 * @param url
	 *            the url to set
	 */
	public void setUrl(String url) {
		this.url = url;
	}

	/**
	 * @return the width
	 */
	public int getWidth() {
		return width;
	}

	/**
	 * @param width
	 *            the width to set
	 */
	public void setWidth(int width) {
		this.width = width;
	}
}
