package common.structures;

import java.util.List;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import common.utils.XMLTools;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 */
public class StateVariable {

	static Logger				logger			= Logger.getLogger(StateVariable.class);

	private boolean				sendingEvents	= false;

	private String				name			= "";

	private String				dataType		= "";

	private String				defaultValue	= "";

	private List<AllowedValue>	allowedValues	= new Vector<AllowedValue>();

	private String				value			= "0";

	private int					minimum			= 0;

	private int					maximum			= 0;

	public int getMaximum() {
		return maximum;
	}

	public void setMaximum(int maximum) {
		this.maximum = maximum;
	}

	public int getMinimum() {
		return minimum;
	}

	public void setMinimum(int minimum) {
		this.minimum = minimum;
	}

	public StateVariable() {

	}

	/**
	 *
	 * @return a XML-Structure (Element) representing this node
	 */
	public Element getXMLElements() {
		Element root = new Element("stateVariable");

		root.setAttribute("sendEvents", XMLTools.boolToYesNo(this.sendingEvents));

		Element name = new Element("name");
		name.setText(this.name);
		root.addContent(name);

		Element dataType = new Element("dataType");
		dataType.setText(this.dataType);
		root.addContent(dataType);

		Element defaultValue = new Element("defaultValue");
		defaultValue.setText(this.defaultValue);

		Element value = new Element("value");
		value.setText(this.value);

		Element allowedValues = new Element("allowedValues");
		for (int i = 0, n = this.allowedValues.size(); i < n; i++) {
			Element allowedValue = new Element(this.allowedValues.get(i).getName());
			allowedValue.setText(this.allowedValues.get(i).getValue());
			allowedValues.addContent(allowedValue);
		}
		if (allowedValues.getChildren().size() > 0)
			root.addContent(allowedValues);

		return root;
	}

	/**
	 *
	 * @return a pretty formated XML String representing this node
	 */
	public String getXMLString() {
		XMLOutputter xmlOut = new XMLOutputter();
		xmlOut.setFormat(Format.getPrettyFormat());
		return xmlOut.outputString(this.getXMLElements());
	}

	/**
	 * @return the allowedValues
	 */
	public List<AllowedValue> getAllowedValues() {
		return allowedValues;
	}

	/**
	 * @param allowedValues
	 *            the allowedValues to set
	 */
	public void setAllowedValues(List<AllowedValue> allowedValues) {
		this.allowedValues = allowedValues;
	}

	/**
	 * @return the dataType
	 */
	public String getDataType() {
		return dataType;
	}

	/**
	 * @param dataType
	 *            the dataType to set
	 */
	public void setDataType(String dataType) {
		this.dataType = dataType;
	}

	/**
	 * @return the defaultValue
	 */
	public String getDefaultValue() {
		return defaultValue;
	}

	/**
	 * @param defaultValue
	 *            the defaultValue to set
	 */
	public void setDefaultValue(String defaultValue) {
		this.defaultValue = defaultValue;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *            the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return the sendingEvents
	 */
	public boolean isSendingEvents() {
		return sendingEvents;
	}

	/**
	 * @param sendingEvents
	 *            the sendingEvents to set
	 */
	public void setSendingEvents(boolean sendingEvents) {
		this.sendingEvents = sendingEvents;
	}

	/**
	 * @return the value
	 */
	public String getValue() {
		return value;
	}

	/**
	 * @param value
	 *            the value to set
	 */
	public void setValue(String value) {
		this.value = value;
	}
}
