/*
 * Created on 31.01.2007
 *
 */
package common.structures;

import org.apache.log4j.Logger;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 * "When Chuck Norris does a pushup, he isnt lifting himself up, hes pushing the Earth down."
 */
public class Subscription {
	static Logger	logger		= Logger.getLogger(Subscription.class);

	private String	callbackUrl	= "";

	private String	sid			= "";

	private long	lastAction	= 0;

	private long	timeOut		= 0;

	private int		seq			= 0;

	/**
	 * Creating a String containig subscription informations
	 */
	public String toString() {
		return "Subscription [callbackUrl: " + this.callbackUrl + " | sid: " + this.sid + " | lastAction: "
				+ this.lastAction + " | timeOut: " + this.timeOut + " | SEQ: " + this.seq + "]";
	}

	/**
	 * @return the seq
	 */
	public synchronized int getSeq() {
		return seq++;
	}

	/**
	 *
	 * @param callbackUrl
	 * @param sid
	 * @param timeOut
	 */
	public Subscription(String callbackUrl, String sid, long timeOut) {
		this.callbackUrl = callbackUrl;
		this.sid = sid;
		this.timeOut = timeOut;
		this.lastAction = System.currentTimeMillis();
	}

	/**
	 *
	 * @return if this subscription has expired
	 */
	public boolean isExpired() {
		return System.currentTimeMillis() - this.lastAction > this.timeOut;
	}

	/**
	 * This modifies the the subscription to the current timestamp.
	 *
	 */
	public void touch() {
		this.lastAction = System.currentTimeMillis();
	}

	/**
	 * @return the callbackUrl
	 */
	public String getCallbackUrl() {
		return callbackUrl;
	}

	/**
	 * @return the sid
	 */
	public String getSid() {
		return sid;
	}

	/**
	 * @return the timeOut in miliseconds
	 */
	public long getTimeOut() {
		return timeOut;
	}

	/**
	 * @param timeOut
	 *            the timeOut to set in miliseconds
	 */
	public void setTimeOut(long timeOut) {
		this.timeOut = timeOut;
	}
}
