/*
 * Created on 12.12.2006
 *
 */
package controlpoint.core;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import common.core.Config;

import controlpoint.soap.SOAPSimpleLightApi;
import controlpoint.structures.RemoteDevice;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author Dominik
 * @author Flo
 *
 * "Chuck Norris hand is the only hand that can beat a Royal Flush."
 */
public class ControlPoint {

	static Logger					logger			= Logger.getLogger(ControlPoint.class);

	private Vector<RemoteDevice>	remoteDevices	= new Vector<RemoteDevice>();

	private RemoteDevice			remoteDevice	= null;

	@SuppressWarnings("unused")
	private Config					conf			= null;

	private int						loadLevel		= 0;

	private int						powerStatus		= 0;

	private boolean					deviceActive	= false;

	private ControlPoint() {

	}

	/**
	 *
	 * @param conf
	 *            Our config
	 */
	public ControlPoint(Config conf) {
		this.conf = conf;
	}

	/**
	 * if the Controlpoint receives a message from the device it requests its attrbutes and saves it
	 *
	 * @param rdev
	 */
	public void handleRemoteDevice(RemoteDevice rdev) {
		String nts = rdev.getNts();
		if (nts.equals("ssdp:alive")) {
			if (addRemoteDevice(rdev)) {
				// step 3,4: request XML file
				Document xml = getXMLforDevice(rdev, "");
				rdev.setXmlSource(xml);
				logger.trace("handleRemoteDevice() -  Location: " + rdev.getLocation());
				SOAPSimpleLightApi ssla = new SOAPSimpleLightApi(rdev);
				logger.trace(ssla.getDimmingLevel());
				logger.trace(ssla.getSwitchStatus());
				try {
					Thread.sleep(200);
				} catch (InterruptedException e) {
					logger.trace(e);
				}
			}

		} else if (nts.equals("ssdp:byebye")) {
			// remove leaving device
			removeRemoteDevice(rdev);
		} else
			logger.trace("Unhandled NTS type Error");
	}

	/**
	 *
	 * @param rdev
	 * @return true by adding the Device succesfully to the controlpoint
	 */
	public boolean addRemoteDevice(RemoteDevice rdev) {
		String rdevUsnID = rdev.getUsnID();
		// add non existing
		if (this.remoteDevice == null || !this.remoteDevice.getUsnID().equalsIgnoreCase(rdevUsnID)) {
			this.remoteDevice = rdev;
			this.deviceActive = true;
			logger.trace("Added Device: " + rdev.getUsnID());
		}
		return true;
	}

	/**
	 *
	 * @param rdev
	 */
	public void removeRemoteDevice(RemoteDevice rdev) {
		this.remoteDevice = null;
		this.deviceActive = false;
		logger.trace("Removed Device: " + rdev.getUsnID());
		return;
	}

	/**
	 *
	 * @param rdev
	 * @param file
	 * @return
	 */
	public static Document getXMLforDevice(RemoteDevice rdev, String file) {
		Document xml = null;
		try {
			if (rdev.getLocation() == null) {
				logger.error("Location Empty in RemoteDevice");
				return null;
			}
			logger.trace("requestXML() - Fetching from Location: " + rdev.getLocation());
			SAXBuilder xmlBuilder = new SAXBuilder();
			// request containing in build:
			try {
				xml = xmlBuilder.build(rdev.getLocation() + file);
			} catch (MalformedURLException e) {
				logger.error("Got MalformedURLException", e);
			}
		} catch (JDOMException e) {
			logger.error("Got JDOMException", e);
		} catch (IOException e) {
			logger.error("Got IOException", e);
		}
		return xml;
	}

	/**
	 *
	 * @return
	 */
	public Vector<RemoteDevice> getRemoteDevices() {
		return remoteDevices;
	}

	/**
	 * @return the deviceActive
	 */
	public boolean isDeviceActive() {
		return deviceActive;
	}

	/**
	 * @return the remoteDevice
	 */
	public RemoteDevice getRemoteDevice() {
		return remoteDevice;
	}

	/**
	 * @return the loadLevel
	 */
	public int getLoadLevel() {
		return loadLevel;
	}

	/**
	 * @param loadLevel
	 *            the loadLevel to set
	 */
	public void setLoadLevel(int loadLevel) {
		this.loadLevel = loadLevel;
	}

	/**
	 * @return the powerStatus
	 */
	public int getPowerStatus() {
		return powerStatus;
	}

	/**
	 * @param powerStatus
	 *            the powerStatus to set
	 */
	public void setPowerStatus(int powerStatus) {
		this.powerStatus = powerStatus;
	}

}
