package controlpoint.gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.WindowConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

import common.core.Config;
import common.http.ConfigHttpd;
import common.http.WebServer;
import common.http.WebServerUDP;
import common.multicast.MulticastListenThread;

import controlpoint.core.ControlPoint;
import controlpoint.multicast.DiscoverThread;
import controlpoint.multicast.EventSubscribeThread;
import controlpoint.multicast.EventSubscriptionRefreshThread;
import controlpoint.multicast.EventUnsubscribeThread;
import controlpoint.soap.SOAPSimpleLightApi;
import controlpoint.soap.SOAPSimpleLightApiRunnerThread;

/**
 *
 * <pre>
 *           Copyright (c) 2007 Dominik Schulz
 *           Copyright (c) 2007 Florian Lindner
 *           Copyright (c) 2007 Philip Hartmann
 *
 *           This file is part of jUPnP.
 *
 *           jUPnP is free software; you can redistribute it and/or modify
 *           it under the terms of the GNU General Public License as published by
 *           the Free Software Foundation; either version 2 of the License, or
 *           (at your option) any later version.
 *
 *           jUPnP is distributed in the hope that it will be useful,
 *           but WITHOUT ANY WARRANTY; without even the implied warranty of
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *           GNU General Public License for more details.
 *
 *           You should have received a copy of the GNU General Public License
 *           along with jUPnP; if not, write to the Free Software
 *           Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author Dominik
 * @author Flo
 *
 * This code was edited or generated using CloudGarden's Jigloo SWT/Swing GUI Builder, which is free for non-commercial
 * use. If Jigloo is being used commercially (ie, by a corporation, company or business for any purpose whatever) then
 * you should purchase a license for each developer using Jigloo. Please visit www.cloudgarden.com for details. Use of
 * Jigloo implies acceptance of these licensing terms. A COMMERCIAL LICENSE HAS NOT BEEN PURCHASED FOR THIS MACHINE, SO
 * JIGLOO OR THIS CODE CANNOT BE USED LEGALLY FOR ANY CORPORATE OR COMMERCIAL PURPOSE.
 *
 * "There is no such thing as global warming. Chuck Norris was cold, so he turned the sun up."
 */
public class ControlPointGui extends javax.swing.JFrame implements ActionListener {
	/**
	 *
	 */
	private static final long				serialVersionUID		= 7766274769423645113L;

	static Logger							logger					= Logger.getLogger(ControlPointGui.class);

	private JPanel							cpPanel;

	private JSlider							dimmerSlider;

	private JLabel							statusLabel;

	private JButton							switchButton;

	private ControlPoint					cp						= null;

	private Config							config					= null;

	private ConfigHttpd						configHttp				= null;

	private JButton							subscribeButton;

	private JButton							subscribeButton2;

	private boolean							dimmingSubButtonClicked	= false;

	private boolean							powerSubButtonClicked	= false;

	private JLabel							powerValueLabel;

	private JLabel							powerLabel;

	private JLabel							dimmerValueLabel;

	private JLabel							dimmerLabel;

	private WebServer						webServer				= null;

	private Thread							webServerThread			= null;

	private WebServerUDP					webServerUdp			= null;

	private Thread							webServerUdpThread		= null;

	private MulticastListenThread			multicastListen			= null;

	private Thread							multicastListenThread	= null;

	private boolean							buttonActivated			= false;

	private DiscoverThread					discover				= null;

	private Thread							discoverThread			= null;

	private EventSubscriptionRefreshThread	dimmerSubscription		= null;

	private Thread							dimmerSubThread			= null;

	private EventSubscriptionRefreshThread	powerSubscription		= null;

	private Thread							powerSubThread			= null;

	/**
	 * Auto-generated main method to display this JFrame
	 */
	public static void main(String[] args) {
		// start the gui
		ControlPointGui inst = new ControlPointGui();
		inst.setAllDisabled();
		inst.setVisible(true);

		while (true) {
			inst.statusLabel.setText(new Integer(inst.cp.getLoadLevel()).toString());
			inst.dimmerValueLabel.setText(new Integer(inst.cp.getLoadLevel()).toString());
			inst.powerValueLabel.setText(new Integer(inst.cp.getPowerStatus()).toString());
			if (inst.cp.isDeviceActive()) {
				inst.setAllEnabled();
			} else {
				inst.setAllDisabled();
			}
			try {
				Thread.sleep(500);
			} catch (InterruptedException e) {
				logger.trace("InterruptedExcepted ", e);
			}
		}
	}

	/**
	 * if there is no device, the ControlPoint shall not invoke any actions
	 *
	 */
	private void setAllDisabled() {
		dimmerSlider.setEnabled(false);
		switchButton.setEnabled(false);
		subscribeButton.setEnabled(false);
		subscribeButton2.setEnabled(false);
		statusLabel.setText("No Device active!");
	}

	/**
	 * if the ControlPoint recognizes a device the ControllPoint shall be able to invoke actions
	 *
	 */
	private void setAllEnabled() {
		dimmerSlider.setEnabled(true);
		switchButton.setEnabled(true);
		subscribeButton.setEnabled(true);
		subscribeButton2.setEnabled(true);
		statusLabel.setText("Device active!");
	}

	public ControlPointGui() {
		super("Light Controller");
		initGUI();
	}

	/**
	 * setup the gui
	 *
	 */
	private void initGUI() {
		try {
			setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
			{
				cpPanel = new JPanel();
				getContentPane().add(cpPanel, BorderLayout.CENTER);
				FlowLayout cpPanelLayout = new FlowLayout();
				cpPanel.setLayout(cpPanelLayout);
				cpPanel.setPreferredSize(new java.awt.Dimension(326, 117));
				{
					dimmerSlider = new JSlider(0, 100);
					dimmerSlider.setMajorTickSpacing(20);
					dimmerSlider.setMinorTickSpacing(2);
					dimmerSlider.setPaintTicks(true);
					dimmerSlider.setPaintTrack(true);
					dimmerSlider.setPaintLabels(true);
					dimmerSlider.addChangeListener(new ChangeListener() {
						public void stateChanged(ChangeEvent e) {
							// a listener for the slider
							int value = ((JSlider) e.getSource()).getValue();
							SOAPSimpleLightApi ssla = new SOAPSimpleLightApi(ControlPointGui.this.cp.getRemoteDevice());
							SOAPSimpleLightApiRunnerThread salat = new SOAPSimpleLightApiRunnerThread(ssla,
									"setDimmingLevel", value);
							Thread th1 = new Thread(salat);
							th1.setPriority(Thread.MIN_PRIORITY);
							th1.start();
						}
					});
					dimmerSlider.addMouseListener(new MouseAdapter() {
						private boolean	clicked	= false;

						public void mouseClicked(MouseEvent e) {
							clicked = true;
						}

						public void mouseReleased(MouseEvent e) {
							if (clicked) {
								clicked = false;
							}
						}
					});
					cpPanel.add(dimmerSlider);
				}
				{
					switchButton = new JButton();
					switchButton.setText("Switch Power");
					switchButton.addActionListener(this);
					cpPanel.add(switchButton);
				}
				{
					subscribeButton2 = new JButton();
					cpPanel.add(subscribeButton2);
					subscribeButton2.setText("^- un-/subscribe (D)");
					subscribeButton2.setPreferredSize(new java.awt.Dimension(188, 22));
					subscribeButton2.addActionListener(new ActionListener() {

						public void actionPerformed(ActionEvent e) {
							if (ControlPointGui.this.dimmingSubButtonClicked) {
								logger
										.trace("ControlPointGui::subscribeButton2::actionPerformed() - dimming Button was clicked! - unsubscribing - SID: "
												+ ControlPointGui.this.cp.getRemoteDevice().getPowerSubscribeSID());
								ControlPointGui.this.dimmingSubButtonClicked = false;
								Thread th1;
								try {
									th1 = new Thread(new EventUnsubscribeThread(ControlPointGui.this.config, new URL(
											ControlPointGui.this.cp.getRemoteDevice().getLocation()
													+ ControlPointGui.this.cp.getRemoteDevice().getService(
															Config.DIMMING_SERVICE_ID).getEventSubURL()),
											ControlPointGui.this.cp.getRemoteDevice().getDimmingSubscribeSID()));
									if (ControlPointGui.this.dimmerSubThread != null)
										ControlPointGui.this.dimmerSubscription.stop();
									th1.start();
								} catch (MalformedURLException e1) {
									e1.printStackTrace();
								}
							} else {
								logger
										.trace("ControlPointGui::subscribeButton2::actionPerformed() - dimming Button was clicked! - subscribing");
								ControlPointGui.this.dimmingSubButtonClicked = true;
								Thread th1 = null;
								try {
									EventSubscribeThread est = new EventSubscribeThread(ControlPointGui.this.config,
											new URL(ControlPointGui.this.cp.getRemoteDevice().getLocation()
													+ ControlPointGui.this.cp.getRemoteDevice().getService(
															Config.DIMMING_SERVICE_ID).getEventSubURL()));
									th1 = new Thread(est);
									th1.start();
									ControlPointGui.this.cp.getRemoteDevice().setDimmingSubscribeSID(est.getSid());
									ControlPointGui.this.dimmerSubscription = new EventSubscriptionRefreshThread(
											ControlPointGui.this.config, new URL(ControlPointGui.this.cp
													.getRemoteDevice().getLocation()
													+ ControlPointGui.this.cp.getRemoteDevice().getService(
															Config.DIMMING_SERVICE_ID).getEventSubURL()),
											ControlPointGui.this.cp.getRemoteDevice().getDimmingSubscribeSID(), 60);
									ControlPointGui.this.dimmerSubThread = new Thread(
											ControlPointGui.this.dimmerSubscription);
									ControlPointGui.this.dimmerSubThread.start();
								} catch (MalformedURLException e1) {
									e1.printStackTrace();
								}
							}
						}

					});
				}
				{
					subscribeButton = new JButton();
					cpPanel.add(subscribeButton);
					subscribeButton.setText("^- un-/subscribe (P)");
					subscribeButton.setPreferredSize(new java.awt.Dimension(98, 22));
					subscribeButton.addActionListener(new ActionListener() {

						public void actionPerformed(ActionEvent e) {
							if (ControlPointGui.this.powerSubButtonClicked) {
								logger
										.trace("ControlPointGui::subscribeButton::actionPerformed() - power Button was clicked! - unsubscribing - SID: "
												+ ControlPointGui.this.cp.getRemoteDevice().getPowerSubscribeSID());
								ControlPointGui.this.powerSubButtonClicked = false;
								Thread th1;
								try {
									th1 = new Thread(new EventUnsubscribeThread(ControlPointGui.this.config, new URL(
											ControlPointGui.this.cp.getRemoteDevice().getLocation()
													+ ControlPointGui.this.cp.getRemoteDevice().getService(
															Config.SWITCH_POWER_SERVICE_ID).getEventSubURL()),
											ControlPointGui.this.cp.getRemoteDevice().getPowerSubscribeSID()));
									th1.start();
									if (ControlPointGui.this.powerSubscription != null)
										ControlPointGui.this.powerSubscription.stop();
								} catch (MalformedURLException e1) {
									e1.printStackTrace();
								}
							} else {
								logger
										.trace("ControlPointGui::subscribeButton::actionPerformed() - power Button was clicked! - subscribing");
								ControlPointGui.this.powerSubButtonClicked = true;
								Thread th1 = null;
								try {
									EventSubscribeThread est = new EventSubscribeThread(ControlPointGui.this.config,
											new URL(ControlPointGui.this.cp.getRemoteDevice().getLocation()
													+ ControlPointGui.this.cp.getRemoteDevice().getService(
															Config.SWITCH_POWER_SERVICE_ID).getEventSubURL()));
									th1 = new Thread(est);
									th1.start();
									ControlPointGui.this.cp.getRemoteDevice().setPowerSubscribeSID(est.getSid());
									ControlPointGui.this.powerSubscription = new EventSubscriptionRefreshThread(
											ControlPointGui.this.config, new URL(ControlPointGui.this.cp
													.getRemoteDevice().getLocation()
													+ ControlPointGui.this.cp.getRemoteDevice().getService(
															Config.SWITCH_POWER_SERVICE_ID).getEventSubURL()),
											ControlPointGui.this.cp.getRemoteDevice().getPowerSubscribeSID(), 60);
									ControlPointGui.this.powerSubThread = new Thread(
											ControlPointGui.this.powerSubscription);
									ControlPointGui.this.powerSubThread.start();
								} catch (MalformedURLException e1) {
									e1.printStackTrace();
								}
							}
						}

					});
				}
				{
					dimmerLabel = new JLabel();
					cpPanel.add(dimmerLabel);
					dimmerLabel.setText("dimmer:");
					dimmerLabel.setPreferredSize(new java.awt.Dimension(71, 14));
				}
				{
					dimmerValueLabel = new JLabel();
					cpPanel.add(dimmerValueLabel);
					dimmerValueLabel.setText("_____");
					dimmerValueLabel.setPreferredSize(new java.awt.Dimension(50, 14));
				}
				{
					powerLabel = new JLabel();
					cpPanel.add(powerLabel);
					powerLabel.setText("Power (0=Off/1=On):");
					powerLabel.setPreferredSize(new java.awt.Dimension(120, 14));
				}
				{
					powerValueLabel = new JLabel();
					cpPanel.add(powerValueLabel);
					powerValueLabel.setText("_____");
					powerValueLabel.setPreferredSize(new java.awt.Dimension(20, 14));
				}
				{
					statusLabel = new JLabel();
					cpPanel.add(statusLabel);
					statusLabel.setText("Status Bar");
					statusLabel.setPreferredSize(new java.awt.Dimension(307, 40));
				}
			}
			pack();
			this.setSize(334, 187);
		} catch (Exception e) {
			logger.trace("Exception occurred ", e);
		}

		BasicConfigurator.configure();
		this.config = new Config();
		this.configHttp = new ConfigHttpd();

		this.cp = new ControlPoint(config);

		// Receving requests, eg NOTIFY
		this.multicastListen = new MulticastListenThread(this.config, this.configHttp, this.cp);
		this.multicastListenThread = new Thread(this.multicastListen);

		this.webServer = WebServer.getWebServerForCP(this.config, this.configHttp, this.cp);
		this.webServerThread = new Thread(this.webServer);

		this.webServerUdp = WebServerUDP.getWebServerUdpForCP(this.config, this.configHttp, this.cp);
		this.webServerUdpThread = new Thread(this.webServerUdp);

		this.discover = new DiscoverThread(this.config);
		this.discoverThread = new Thread(this.discover);

		this.webServerUdpThread.start();
		this.webServerThread.start();
		this.multicastListenThread.start();
		this.discoverThread.start();
	}

	public void actionPerformed(ActionEvent evt) {
		Object source = evt.getSource();
		if (source == switchButton) {
			statusLabel.setText("Button Pressed");
			SOAPSimpleLightApi ssla = new SOAPSimpleLightApi(this.cp.getRemoteDevice());
			SOAPSimpleLightApiRunnerThread salat = null;
			if (!this.buttonActivated) {
				salat = new SOAPSimpleLightApiRunnerThread(ssla, "switchOn", 0);
				this.buttonActivated = true;
			} else {
				salat = new SOAPSimpleLightApiRunnerThread(ssla, "switchOff", 0);
				this.buttonActivated = false;
			}
			Thread th1 = new Thread(salat);
			th1.setPriority(Thread.MIN_PRIORITY);
			th1.start();
		} else if (source == dimmerSlider) {
			statusLabel.setText("Slider used");
		} else
			logger.error("Handling UI Actions failed!");
		repaint();
	}

}
