/*
 * Created on 12.01.2007
 *
 */
package controlpoint.multicast;

import java.io.IOException;
import java.net.URL;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;

import common.core.Config;
import common.http.CustomHttpRequest;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 * "Chuck Norris gave Mona Lisa that smile."
 */
public class EventSubscribeThread implements Runnable {

	static Logger	logger	= Logger.getLogger(EventSubscribeThread.class);

	private Config	config	= null;

	private URL		target	= null;

	private String	sid		= "";

	/**
	 *
	 * @param conf
	 *            Our Config
	 * @param url
	 *            The Control-URL to subscribe to
	 */
	public EventSubscribeThread(Config conf, URL url) {
		this.config = conf;
		this.target = url;
	}

	/**
	 * statrting the eventsubscribe-thread
	 */
	public void run() {
		try {
			// create and send a subscribe request
			CustomHttpRequest req = new CustomHttpRequest(this.target, "SUBSCRIBE");
			req.addHeaderLine("Timeout: Second-60");
			logger.trace("Config-Local Address: " + this.config.getLocalAddress().toString());
			logger.trace("Config HTTP Server Port: " + this.config.getHttpServerPort());
			req.addHeaderLine("Callback: <http://" + new String(this.config.getLocalAddress().getHostAddress()) + ":"
					+ this.config.getHttpServerPort() + ">");
			req.addHeaderLine("NT: upnp:event");
			logger.trace(req.toString());
			req.sendRequest();
			StringTokenizer tokens = new StringTokenizer(req.getResponseHeader(), "\n");
			while (tokens.hasMoreTokens()) {
				String buffer = tokens.nextToken();
				if (buffer.toUpperCase().startsWith("SID:".toUpperCase())) {
					this.sid = buffer.substring(5);
					logger.trace("SID: " + this.sid);
				}
			}
			logger.trace("EventSubscribeThread-Response-Header: " + req.getResponseHeader());
			logger.trace("EventSubscribeThread-Response-Body: " + req.getResponse());
		} catch (IOException e) {
			logger.trace("Got IOException", e);
		}

	}

	/**
	 * @return the sid
	 */
	public String getSid() {
		try {
			Thread.sleep(1000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		return sid;
	}

}
