/*
 * Created on 31.01.2007
 *
 */
package controlpoint.multicast;

import java.io.IOException;
import java.net.URL;

import org.apache.log4j.Logger;

import common.core.Config;
import common.http.CustomHttpRequest;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 * "Chuck Norris can slam a revolving door."
 */
public class EventSubscriptionRefreshThread implements Runnable {

	static Logger	logger			= Logger.getLogger(EventSubscriptionRefreshThread.class);

	private String	sid				= "";

	private URL		url				= null;

	@SuppressWarnings("unused")
	private Config	config			= null;

	private long	refeshTimeout	= 10 * 1000;												// default: 10 second

	// refresh

	private boolean	running			= false;

	/**
	 *
	 * @param config
	 * @param url
	 * @param sid
	 * @param refresh
	 *            Refresh Timeout in seconds
	 */
	public EventSubscriptionRefreshThread(Config config, URL url, String sid, int refresh) {
		this.sid = sid;
		this.config = config;
		this.url = url;
		this.refeshTimeout = refresh * 1000;
		this.running = true;
	}

	/**
	 * Stop this Thread. This sets a bool to false. It's checked on every loop of this thread.
	 *
	 */
	public void stop() {
		this.running = false;
	}

	public void run() {
		while (this.running) {
			try {
				// create and send a subscribe (renewal) request
				CustomHttpRequest req = new CustomHttpRequest(this.url, "SUBSCRIBE");
				req.addHeaderLine("Timeout: Second-60");
				req.addHeaderLine("SID: " + this.sid);
				logger.trace(req.toString());
				req.sendRequest();
			} catch (IOException e) {
				logger.trace(e);
			}
			try {
				Thread.sleep(this.refeshTimeout);
			} catch (InterruptedException e) {
				logger.trace(e);
			}
		}
	}
}
