/*
 * Created on 11.01.2007
 *
 */
package controlpoint.soap;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.SocketException;
import java.net.URL;

import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

import common.core.Config;
import common.http.CustomHttpRequest;
import common.utils.XMLTools;

import controlpoint.structures.RemoteDevice;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 * "Remember the Soviet Union? They decided to quit after watching a DeltaForce marathon on Satellite TV."
 */
public class SOAPSimpleLightApi {

	static Logger			logger	= Logger.getLogger(SOAPSimpleLightApi.class);

	private RemoteDevice	rdev	= null;

	public SOAPSimpleLightApi(RemoteDevice rdev) {
		this.rdev = rdev;
	}

	/**
	 * A convenience method for constructing the control url
	 *
	 * @return
	 */
	private String getSwitchCtrlUrl() {
		return this.rdev.getLocation() + this.rdev.getService(Config.SWITCH_POWER_SERVICE_ID).getControlURL();
	}

	/**
	 * A convenience method for constructing the control url
	 *
	 * @return
	 */
	private String getDimmingCtrlUrl() {
		return this.rdev.getLocation() + this.rdev.getService(Config.DIMMING_SERVICE_ID).getControlURL();
	}

	/**
	 * turn on the device light
	 *
	 */
	public void switchOn() {
		try {
			// create a HTTP-POST-request
			URL url = new URL(this.getSwitchCtrlUrl());
			CustomHttpRequest req = new CustomHttpRequest(url, "POST");
			req.addHeaderLine("Content-Type: text/xml; charset=\"utf-8\"");
			req.addHeaderLine("SOAPACTION: \"urn:schemas-upnp-org:service:SwitchPower:1#SetTarget\"");
			// harcoded payload
			String payload = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n"
					+ "<s:Envelope s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\">\r\n"
					+ "   <s:Body>\r\n"
					+ "      <u:SetTarget xmlns:u=\"urn:schemas-upnp-org:service:SwitchPower:1\">\r\n"
					+ "         <newTargetValue>1</newTargetValue>\r\n" + "      </u:SetTarget>\r\n"
					+ "   </s:Body>\r\n" + "</s:Envelope>";
			req.setPayload(payload);
			try {
				req.sendRequest();
			} catch (SocketException e) {
				logger.trace("Got SocketException", e);
			}
			logger.trace("--- ### --- Response-Header:\n" + req.getResponseHeader());
			logger.trace("--- ### --- Response:\n" + req.getResponse());
		} catch (IOException e) {
			logger.trace("IOException occurred ", e);
		}
	}

	/**
	 * switchoff device light
	 *
	 */
	public void switchOff() {
		try {
			URL url = new URL(this.getSwitchCtrlUrl());
			// POST Http request
			CustomHttpRequest req = new CustomHttpRequest(url, "POST");
			req.addHeaderLine("Content-Type: text/xml; charset=\"utf-8\"");
			req.addHeaderLine("SOAPACTION: \"urn:schemas-upnp-org:service:SwitchPower:1#SetTarget\"");
			String payload = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n"
					+ "<s:Envelope s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\">\r\n"
					+ "   <s:Body>\r\n"
					+ "      <u:SetTarget xmlns:u=\"urn:schemas-upnp-org:service:SwitchPower:1\">\r\n"
					+ "         <newTargetValue>0</newTargetValue>\r\n" + "      </u:SetTarget>\r\n"
					+ "   </s:Body>\r\n" + "</s:Envelope>";
			req.setPayload(payload);
			req.sendRequest();
			logger.trace("--- ### --- Response-Header:\n" + req.getResponseHeader());
			logger.trace("--- ### --- Response:\n" + req.getResponse());
		} catch (IOException e) {
			logger.trace("IOException occurred ", e);
		}
	}

	/**
	 *
	 * @return wheter the light is turnend on or not
	 */
	public String getSwitchStatus() {
		try {
			URL url = new URL(this.getSwitchCtrlUrl());
			CustomHttpRequest req = new CustomHttpRequest(url, "POST");
			req.addHeaderLine("Content-Type: text/xml; charset=\"utf-8\"");
			req.addHeaderLine("SOAPACTION: \"urn:schemas-upnp-org:service:SwitchPower:1#SetTarget\"");
			String payload = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n"
					+ "<s:Envelope s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\">\r\n"
					+ "   <s:Body>\r\n"
					+ "      <u:GetStatus xmlns:u=\"urn:schemas-upnp-org:service:SwitchPower:1\" />\r\n"
					+ "   </s:Body>\r\n" + "</s:Envelope>";
			req.setPayload(payload);
			req.sendRequest();
			logger.trace("Response-Header:\n" + req.getResponseHeader());
			logger.trace("Response:\n" + req.getResponse());
			Document xml = null;
			try {
				if (rdev.getLocation() == null) {
					logger.error("Location Empty in RemoteDevice");
					return null;
				}
				logger.trace("requestXML() - Fetching from Location: " + rdev.getLocation());
				// SAXBuilder creats XML-Content
				SAXBuilder xmlBuilder = new SAXBuilder();
				// request containing in build:
				ByteArrayInputStream bais = new ByteArrayInputStream(req.getResponse().getBytes());
				xml = xmlBuilder.build(bais);
				logger.trace(xml.toString());
				logger.trace("ResultStatus: "
						+ XMLTools.getXMLNode(xml.getRootElement().getChildren(), "ResultStatus").getText());
				return XMLTools.getXMLNode(xml.getRootElement().getChildren(), "ResultStatus").getText();
			} catch (JDOMException e) {
				logger.trace("Got a JDOMException", e);
			} catch (IOException e) {
				logger.error("Got a IOException", e);
			}
		} catch (IOException e) {
			logger.trace("IOException occurred ", e);
		}
		return "";
	}

	/**
	 * Set the dimming level
	 *
	 * @param level
	 */
	public void setDimmingLevel(int level) {
		try {
			URL url = new URL(this.getDimmingCtrlUrl());
			CustomHttpRequest req = new CustomHttpRequest(url, "POST");
			// hard coded
			req.addHeaderLine("Content-Type: text/xml; charset=\"utf-8\"");
			req.addHeaderLine("SOAPACTION: \"urn:schemas-upnp-org:service:DimmingService:1#SetLoadLevelTarget\"");
			String payload = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n"
					+ "<s:Envelope s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\">\r\n"
					+ "   <s:Body>\r\n"
					+ "      <u:SetLoadLevelTarget xmlns:u=\"urn:schemas-upnp-org:service:DimmingService:1\">\r\n"
					+ "         <NewLoadLevelTarget>" + level + "</NewLoadLevelTarget>\r\n"
					+ "      </u:SetLoadLevelTarget>\r\n" + "   </s:Body>\r\n" + "</s:Envelope>";
			req.setPayload(payload);
			try {
				req.sendRequest();
			} catch (SocketException e) {
				logger.trace("Got SocketException", e);
			}
			logger.trace("Response-Header:\n" + req.getResponseHeader());
			logger.trace("Response:\n" + req.getResponse());
		} catch (IOException e) {
			logger.trace("Got a IOException", e);
		}
	}

	/**
	 * get the dimminglevel from the jLight
	 * @return
	 */
	public int getDimmingLevel() {
		try {
			URL url = new URL(this.getDimmingCtrlUrl());
			CustomHttpRequest req = new CustomHttpRequest(url, "POST");
			req.addHeaderLine("Content-Type: text/xml; charset=\"utf-8\"");
			req.addHeaderLine("SOAPACTION: \"urn:schemas-upnp-org:service:DimmingService:1#GetLoadLevelStatus\"");
			String payload = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n"
					+ "<s:Envelope s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\" xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\">\r\n"
					+ "   <s:Body>\r\n"
					+ "      <u:GetLoadLevelStatus xmlns:u=\"urn:schemas-upnp-org:service:DimmingService:1\" />\r\n"
					+ "   </s:Body>\r\n" + "</s:Envelope>";
			req.setPayload(payload);
			req.sendRequest();
			logger.trace("Response-Header:\n" + req.getResponseHeader());
			logger.trace("Response:\n" + req.getResponse());
			Document xml = null;
			try {
				if (rdev.getLocation() == null) {
					logger.error("Location Empty in RemoteDevice");
					return 0;
				}
				logger.trace("requestXML() - Fetching from Location: " + rdev.getLocation());
				SAXBuilder xmlBuilder = new SAXBuilder();
				// request containing in build:
				ByteArrayInputStream bais = new ByteArrayInputStream(req.getResponse().getBytes());
				xml = xmlBuilder.build(bais);
				logger.trace(xml.toString());
				int loadLevelStatus = 0;
				try {
					loadLevelStatus = Integer.parseInt(XMLTools.getXMLNode(xml.getRootElement().getChildren(),
							"RetLoadLevelStatus").getText());
					logger.trace("ResultStatus: " + loadLevelStatus);
				} catch (NullPointerException e) {
					logger.trace("NullPointerException occured ", e);
				}
				return loadLevelStatus;
			} catch (JDOMException e) {
				logger.trace("Got a JDOMException", e);
			} catch (IOException e) {
				logger.trace("Got a IOException", e);
			}
		} catch (IOException e) {
			logger.trace("Got a IOException", e);
		}
		return 0;
	}
}
