package controlpoint.structures;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;

import common.core.Config;
import common.structures.Icon;
import common.structures.Service;
import controlpoint.core.ControlPoint;

import device.structures.Device;

/**
 * <pre>
 *           Copyright (c) 2007 Dominik Schulz
 *           Copyright (c) 2007 Florian Lindner
 *           Copyright (c) 2007 Philip Hartmann
 *
 *           This file is part of jUPnP.
 *
 *           jUPnP is free software; you can redistribute it and/or modify
 *           it under the terms of the GNU General Public License as published by
 *           the Free Software Foundation; either version 2 of the License, or
 *           (at your option) any later version.
 *
 *           jUPnP is distributed in the hope that it will be useful,
 *           but WITHOUT ANY WARRANTY; without even the implied warranty of
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *           GNU General Public License for more details.
 *
 *           You should have received a copy of the GNU General Public License
 *           along with jUPnP; if not, write to the Free Software
 *           Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 * "Some people wear Superman pajamas. Superman wears Chuck Norris pajamas."
 */
public class RemoteDevice extends Device {

	static Logger				logger				= Logger.getLogger(RemoteDevice.class);

	/**
	 * The value of the Content-Length: header field
	 */
	private int					contentLength		= 0;

	/**
	 * The value of the Host: header field
	 */
	private String				host				= "";

	public static final String	locationStr			= "LOCATION:";

	private String				location			= "";

	private Document			xmlSource			= null;

	private List<Icon>			iconList			= new ArrayList<Icon>();

	public static final String	ntStr				= "NT:";

	private String				nt					= "";

	public static final String	ntsStr				= "NTS:";

	private String				nts					= "";

	public static final String	serverStr			= "SERVER:";

	private String				server				= "";

	private String				powerSubscribeSID	= "";

	private String				dimmingSubscribeSID	= "";

	public static final String	usnStr				= "USN:";

	private String				usn					= "";

	private String				usnID				= "";

	public static final String	cacheControlStr		= "CACHE-CONTROL:";

	private String				cacheControl		= "";

	public static final String	stStr				= "ST:";

	private String				st					= "";

	public static final String	mxStr				= "MX:";

	private String				mx					= "";

	public static final String	manStr				= "MAN:";

	private String				man					= "";

	public static final String	sidStr				= "SID:";

	private String				sid					= "";

	public static final String	subscriptionIdStr	= "Subscription-ID:";

	private String				subscriptionId		= "";

	public static final String	callBackStr			= "Call-Back:";

	private String				callBack			= "";

	public static final String	ScopeStr			= "Scope:";

	private String				scope				= "";

	public static final String	timeOutStr			= "Timeout:";

	private String				timeOut				= "";

	public static final String	optStr				= "OPT:";

	private String				opt					= "";

	public static final String	o1nlsStr			= "01-NLS_";

	public RemoteDevice(boolean isRootdevice, Config conf) {
		super(isRootdevice, conf);
	}

	public static String getCacheControlStr() {
		return cacheControlStr;
	}

	public static String getCallBackStr() {
		return callBackStr;
	}

	public static String getLocationStr() {
		return locationStr;
	}

	public static String getManStr() {
		return manStr;
	}

	public static String getMxStr() {
		return mxStr;
	}

	public static String getNtsStr() {
		return ntsStr;
	}

	public static String getNtStr() {
		return ntStr;
	}

	public static String getO1nlsStr() {
		return o1nlsStr;
	}

	public static String getOptStr() {
		return optStr;
	}

	public static String getScopeStr() {
		return ScopeStr;
	}

	public static String getServerStr() {
		return serverStr;
	}

	public static String getSidStr() {
		return sidStr;
	}

	public static String getStStr() {
		return stStr;
	}

	public static String getSubscriptionIdStr() {
		return subscriptionIdStr;
	}

	public static String getTimeOutStr() {
		return timeOutStr;
	}

	public static String getUsnStr() {
		return usnStr;
	}

	public String getCacheControl() {
		return cacheControl;
	}

	public void setCacheControl(String cacheControl) {
		this.cacheControl = cacheControl;
	}

	public String getCallBack() {
		return callBack;
	}

	public void setCallBack(String callBack) {
		this.callBack = callBack;
	}

	public int getContentLength() {
		return contentLength;
	}

	public void setContentLength(int contentLength) {
		this.contentLength = contentLength;
	}

	public String getHost() {
		return host;
	}

	public void setHost(String host) {
		this.host = host;
	}

	public String getLocation() {
		return location;
	}

	public void setLocation(String location) {
		this.location = location;
	}

	public String getMan() {
		return man;
	}

	public void setMan(String man) {
		this.man = man;
	}

	public String getMx() {
		return mx;
	}

	public void setMx(String mx) {
		this.mx = mx;
	}

	public String getNt() {
		return nt;
	}

	public void setNt(String nt) {
		this.nt = nt;
	}

	public String getNts() {
		return nts;
	}

	public void setNts(String nts) {
		this.nts = nts;
	}

	public String getOpt() {
		return opt;
	}

	public void setOpt(String opt) {
		this.opt = opt;
	}

	public String getScope() {
		return scope;
	}

	public void setScope(String scope) {
		this.scope = scope;
	}

	public String getServer() {
		return server;
	}

	public void setServer(String server) {
		this.server = server;
	}

	public String getSid() {
		return sid;
	}

	public void setSid(String sid) {
		this.sid = sid;
	}

	public String getSt() {
		return st;
	}

	public void setSt(String st) {
		this.st = st;
	}

	public String getSubscriptionId() {
		return subscriptionId;
	}

	public void setSubscriptionId(String subscriptionId) {
		this.subscriptionId = subscriptionId;
	}

	public String getTimeOut() {
		return timeOut;
	}

	public void setTimeOut(String timeOut) {
		this.timeOut = timeOut;
	}

	public String getUsn() {
		return usn;
	}

	public void setUsn(String usn) {
		this.usn = usn;
		if (!usn.contains("::")) {
			this.usnID = usn;
		} else {
			this.usnID = usn.substring(0, usn.indexOf("::"));
		}
	}

	public String getUsnID() {
		return usnID;
	}

	public Document getXmlSource() {
		return xmlSource;
	}

	/**
	 * parsing the XML-tree and add data to the device
	 * @param xmlSource
	 */
	public void setXmlSource(Document xmlSource) {
		this.xmlSource = xmlSource;
		List mainNodes = this.getXmlSource().getRootElement().getChildren();
		for (Iterator iter = mainNodes.iterator(); iter.hasNext();) {
			Element element = (Element) iter.next();
			if (element == null) {
				continue;
			}
			logger.trace(element.toString());
			// get to the device entrys
			if (element.getName().equalsIgnoreCase("device")) {
				List deviceNodes = element.getChildren();
				for (Iterator iter2 = deviceNodes.iterator(); iter2.hasNext();) {
					Element element2 = (Element) iter2.next();
					if (element2 == null) {
						continue;
					}
					logger.trace("Element 2:" + element2.toString());
					// get to the serviceList entrys
					if (element2.getName().equalsIgnoreCase("serviceList")) {
						List serivceNodes = element2.getChildren();
						for (Iterator iter3 = serivceNodes.iterator(); iter3.hasNext();) {
							Element element3 = (Element) iter3.next();
							if (element3 == null) {
								continue;
							}
							Service serv = new Service(super.getConfig());
							logger.trace("Element 3: " + element3.toString());
							List servicesInServiceList = element3.getChildren();
							for (Iterator iter4 = servicesInServiceList.iterator(); iter4.hasNext();) {
								Element service = (Element) iter4.next();
								if (service == null) {
									continue;
								}
								// logger.trace("Service: " + service);
								if (service.getName().equalsIgnoreCase("serviceType")) {
									logger.trace("ServiceType: " + service.getText());
									serv.setServiceType(service.getText());
								}
								if (service.getName().equalsIgnoreCase("serviceID")) {
									logger.trace("ServiceID: " + service.getText());
									serv.setServiceID(service.getText());
								}
								if (service.getName().equalsIgnoreCase("SCPDURL")) {
									logger.trace("SCDP-URL: " + service.getText());
									serv.setDescriptionURL(service.getText());
								}
								if (service.getName().equalsIgnoreCase("controlURL")) {
									logger.trace("controlURL: " + service.getText());
									serv.setControlURL(service.getText());
								}
								if (service.getName().equalsIgnoreCase("eventSubURL")) {
									logger.trace("eventSub: " + service.getText());
									serv.setEventSubURL(service.getText());
								}
							}
							serv.setXmlSource(ControlPoint.getXMLforDevice(this, serv.getDescriptionURL()));
							// this.serviceList.add(serv);
							super.addService(serv);
							logger.trace("Service: " + serv.getDescriptionURL());
						}
					}
				}
			}
		}
	}

	public List<Icon> getIconList() {
		return iconList;
	}

	public String getPowerSubscribeSID() {
		return powerSubscribeSID;
	}

	public void setPowerSubscribeSID(String subscribeSID) {
		this.powerSubscribeSID = subscribeSID;
	}

	public String getDimmingSubscribeSID() {
		return dimmingSubscribeSID;
	}

	public void setDimmingSubscribeSID(String subscribeSID) {
		this.dimmingSubscribeSID = subscribeSID;
	}

}
