package device.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;
import java.util.Random;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.WindowConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

import common.core.Config;
import common.core.HeartbeatThread;
import common.http.ConfigHttpd;
import common.http.WebServer;
import common.http.WebServerUDP;
import common.multicast.MulticastListenThread;
import common.structures.Action;
import common.structures.AllowedValue;
import common.structures.Argument;
import common.structures.Icon;
import common.structures.Service;
import common.structures.StateVariable;

import device.structures.Device;

/**
 * /**
 *
 * <pre>
 *            Copyright (c) 2007 Dominik Schulz
 *            Copyright (c) 2007 Florian Lindner
 *            Copyright (c) 2007 Philip Hartmann
 *
 *            This file is part of jUPnP.
 *
 *            jUPnP is free software; you can redistribute it and/or modify
 *            it under the terms of the GNU General Public License as published by
 *            the Free Software Foundation; either version 2 of the License, or
 *            (at your option) any later version.
 *
 *            jUPnP is distributed in the hope that it will be useful,
 *            but WITHOUT ANY WARRANTY; without even the implied warranty of
 *            MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *            GNU General Public License for more details.
 *
 *            You should have received a copy of the GNU General Public License
 *            along with jUPnP; if not, write to the Free Software
 *            Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author Dominik
 * @author Flo
 *
 * This code was edited or generated using CloudGarden's Jigloo SWT/Swing GUI Builder, which is free for non-commercial
 * use. If Jigloo is being used commercially (ie, by a corporation, company or business for any purpose whatever) then
 * you should purchase a license for each developer using Jigloo. Please visit www.cloudgarden.com for details. Use of
 * Jigloo implies acceptance of these licensing terms. A COMMERCIAL LICENSE HAS NOT BEEN PURCHASED FOR THIS MACHINE, SO
 * JIGLOO OR THIS CODE CANNOT BE USED LEGALLY FOR ANY CORPORATE OR COMMERCIAL PURPOSE.
 *
 * "If, by some incredible space-time paradox, Chuck Norris would ever fight himself, he'd win. Period."
 */
public class DeviceGui extends javax.swing.JFrame implements ActionListener {

	/**
	 *
	 */
	private static final long		serialVersionUID		= -4544790517301440528L;

	static Logger					logger					= Logger.getLogger(DeviceGui.class);

	private Config					config					= null;

	private ConfigHttpd				configHttpd				= null;

	private WebServer				webServer				= null;

	private Thread					webServerThread			= null;

	private WebServerUDP			webServerUdp			= null;

	private Thread					webServerUdpThread		= null;

	private MulticastListenThread	multicastListen			= null;

	private Thread					multicastListenThread	= null;

	private Thread					heartbeatThread			= null;

	private JPanel					mainPanel;

	private JLabel					powerValueLabel;

	private JLabel					statusBarLabel;

	private JLabel					dimmerValueLabel;

	private JLabel					powerLabel;

	private JButton					powerButton;

	private JLabel					dimmerLabel;

	private JSlider					dimmerSlider;

	/**
	 * Auto-generated main method to display this JFrame
	 */
	public static void main(String[] args) {
		BasicConfigurator.configure();
		DeviceGui inst = new DeviceGui();
		inst.setVisible(true);
	}

	public DeviceGui() {
		super("UPnP Device: jLight");
		this.initUpnpDevice();
		initGUI();
		UpdateThread ut = this.new UpdateThread(this);
		Thread t = new Thread(ut);
		t.start();
	}

	/**
	 * initialize the GUI
	 *
	 */
	private void initGUI() {
		try {
			setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
			addWindowListener(new WindowListener() {

				public void windowActivated(WindowEvent e) {
					// not necessary
				}

				@SuppressWarnings("deprecation")
				public void windowClosed(WindowEvent e) {
					// Close our Sockets and shut down the http-server
					DeviceGui.this.webServerThread.stop();
					DeviceGui.this.webServerUdpThread.stop();
					DeviceGui.this.multicastListenThread.stop();
					DeviceGui.this.config.close();
				}

				/**
				 * When the device is closed, it has to send a "byebye"
				 */
				public void windowClosing(WindowEvent e) {
					// Send an SSDP:ByeBye Message when exiting.
					DeviceGui.this.config.getDevice().sendSSDPByeBye();
					try {
						Thread.sleep(10);
					} catch (InterruptedException e1) {
						logger.trace("InterruptedException occurred ", e1);
					}
					DeviceGui.this.config.getDevice().sendSSDPByeBye();
				}

				public void windowDeactivated(WindowEvent e) {
					// not necessary

				}

				public void windowDeiconified(WindowEvent e) {
					// not necessary

				}

				public void windowIconified(WindowEvent e) {
					// not necessary
				}

				public void windowOpened(WindowEvent e) {
					// not necessary
				}

			});
			{
				mainPanel = new JPanel();
				getContentPane().add(mainPanel, BorderLayout.CENTER);
				GridBagLayout mainPanelLayout = new GridBagLayout();
				mainPanelLayout.rowWeights = new double[] { 0.0, 0.0, 0.1 };
				mainPanelLayout.rowHeights = new int[] { 27, 36, 7 };
				mainPanelLayout.columnWeights = new double[] { 0.0, 0.1, 0.1 };
				mainPanelLayout.columnWidths = new int[] { 225, 7, 7 };
				mainPanel.setLayout(mainPanelLayout);
				{
					dimmerSlider = new JSlider(0, 100);
					dimmerSlider.setMajorTickSpacing(20);
					dimmerSlider.setMinorTickSpacing(2);
					dimmerSlider.setPaintTicks(true);
					dimmerSlider.setPaintTrack(true);
					dimmerSlider.setPaintLabels(true);
					dimmerSlider.addChangeListener(new ChangeListener() {

						public void stateChanged(ChangeEvent e) {
							int value = ((JSlider) e.getSource()).getValue();
							DeviceGui.this.config.getDevice().getService(Config.DIMMING_SERVICE_ID).setStateVariable(
									"LoadLevelStatus", new Integer(value).toString());
							DeviceGui.this.dimmerValueLabel.setText(new Integer(value).toString());
						}
					});
					mainPanel.add(dimmerSlider, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
							GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
				}
				{
					dimmerLabel = new JLabel();
					mainPanel.add(dimmerLabel, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
							GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
					dimmerLabel.setText("dimmer:");
				}
				{
					powerButton = new JButton();
					mainPanel.add(powerButton, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
							GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
					powerButton.setText("Switch Power");
					powerButton.addActionListener(new ActionListener() {

						public void actionPerformed(ActionEvent e) {
							if (DeviceGui.this.config.getDevice().getService(Config.SWITCH_POWER_SERVICE_ID)
									.getStateVariableValue("Status").equalsIgnoreCase("1")) {
								DeviceGui.this.config.getDevice().getService(Config.SWITCH_POWER_SERVICE_ID)
										.setStateVariable("Status", "0");
								DeviceGui.this.powerValueLabel.setText("Off");
							} else {
								DeviceGui.this.config.getDevice().getService(Config.SWITCH_POWER_SERVICE_ID)
										.setStateVariable("Status", "1");
								DeviceGui.this.powerValueLabel.setText("On");
							}
						}

					});
				}
				{
					powerLabel = new JLabel();
					mainPanel.add(powerLabel, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
							GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
					powerLabel.setText("power:");
				}
				{
					dimmerValueLabel = new JLabel();
					mainPanel.add(dimmerValueLabel, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
							GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
					dimmerValueLabel.setText("dimmerValue");
				}
				{
					powerValueLabel = new JLabel();
					mainPanel.add(powerValueLabel, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0,
							GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
					powerValueLabel.setText("powerValue");
				}
				{
					statusBarLabel = new JLabel();
					mainPanel.add(statusBarLabel, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
							GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
					statusBarLabel.setText("StatusBar");
				}
			}
			pack();
			this.setSize(400, 135);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void actionPerformed(ActionEvent evt) {
		Object source = evt.getSource();
		if (source == powerButton) {
			statusBarLabel.setText("Switch power toggled");

		}
	}

	/**
	 * update the gui corresponding the controlpoint
	 *
	 */
	private class UpdateThread implements Runnable {

		private DeviceGui	devguy	= null;

		public UpdateThread(DeviceGui devguy) {
			this.devguy = devguy;
		}

		public void run() {
			while (true) {
				// if the light is on set the text in the gui
				if (this.devguy.config.getDevice().getService(Config.SWITCH_POWER_SERVICE_ID).getStateVariableValue(
						"Status").equalsIgnoreCase("1")) {
					this.devguy.powerValueLabel.setText("On");
				} else {
					this.devguy.powerValueLabel.setText("Off");
				}
				// set the dimmervalue in the gui
				this.devguy.dimmerValueLabel.setText(this.devguy.config.getDevice().getService(
						Config.DIMMING_SERVICE_ID).getStateVariableValue("LoadLevelStatus"));
				this.devguy.dimmerSlider.setValue(Integer.parseInt(this.devguy.config.getDevice().getService(
						Config.DIMMING_SERVICE_ID).getStateVariableValue("LoadLevelStatus")));
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					logger.trace("InterruptedException occurred ", e);
				}
			}
		}
	}

	/**
	 * Creates a virtual Device
	 */
	private void initUpnpDeviceStructure() {
		// setting up fictional values similar to the intel light
		Device dev = new Device(true, this.config);
		dev.setDeviceType("urn:schemas-upnp-org:device:BinaryLight:1");
		dev.setFriendlyName("jLight");
		dev.setManufacturer("TU Darmstadt - FB Informatik");
		try {
			dev.setManufacturerURL(new URL("http://www.informatik.tu-darmstadt.de/"));
			dev.setModelURL(new URL("http://developer.gauner.org/jlight/"));
			dev.setDescriptionURL(new URL("http://developer.gauner.org/jlight/desc"));
		} catch (MalformedURLException e) {
			logger.trace("MalformedURLException occurred ", e);
		}
		dev.setModelDescription("Software Emulated Light Bulb - I'll be back");
		dev.setModelName("TUD CLR Emulated Light Bulb");
		dev.setModelNumber("T-2000");
		dev.setUDN("uuid:" + this.config.getUuid());
		// init the services
		this.initUpnpServiceStructure(dev);
		// add the icon
		Icon iconForDevice = new Icon();
		iconForDevice.setMimeType("image/png");
		iconForDevice.setWidth(32);
		iconForDevice.setHeight(32);
		iconForDevice.setDepth(8);
		iconForDevice.setUrl("/icon.png");
		// add the icon to our config
		this.config.setDevice(dev);
	}

	/**
	 * initialize services
	 * @param dev
	 */
	private void initUpnpServiceStructure(Device dev) {
		{
			// ############## DIMMING SERVICE ################
			// set up the DimmingService
			Service dimServ = new Service(this.config);
			dimServ.setServiceType(Config.DIMMING_SERVICE_TYPE);
			// dimServ.setServiceID("urn:schemas-upnp-org:serviceId:DimmingService.0001");
			dimServ.setServiceID(Config.DIMMING_SERVICE_ID);
			// set up the Actions for the Dimming Service
			// set up the action GetLoadLevelStatus
			Action actGetLoadLevelStatus = new Action();
			actGetLoadLevelStatus.setName("GetLoadLevelStatus");
			// set up the arguments for getloadlevelstatus
			Argument argRetLoadLevelStatus = new Argument();
			argRetLoadLevelStatus.setName("RetLoadLevelStatus");
			argRetLoadLevelStatus.setDirection("out");
			StateVariable stateLoadLevelStatus = new StateVariable();
			stateLoadLevelStatus.setName("LoadLevelStatus");
			argRetLoadLevelStatus.setRelatedStateVariable(stateLoadLevelStatus);
			// end
			actGetLoadLevelStatus.addArgument(argRetLoadLevelStatus);
			// end
			dimServ.addAction(actGetLoadLevelStatus);
			// set up the action GetMinLevel
			Action actGetMinLevel = new Action();
			actGetMinLevel.setName("GetMinLevel");
			// set up the arguments for getminlevel
			Argument argMinLevel = new Argument();
			argMinLevel.setName("MinLevel");
			argMinLevel.setDirection("out");
			StateVariable stateVarMinLevel = new StateVariable();
			stateVarMinLevel.setName("MinLevel");
			argMinLevel.setRelatedStateVariable(stateVarMinLevel);
			// end
			actGetMinLevel.addArgument(argMinLevel);
			// end
			dimServ.addAction(actGetMinLevel);
			// set up the action SetLoadLevelTarget
			Action actSetLoadLevelTarget = new Action();
			actSetLoadLevelTarget.setName("SetLoadLevelTarget");
			// set up the arguments for setloadleveltarget
			Argument argNewLoadLevelTarget = new Argument();
			argNewLoadLevelTarget.setName("NewLoadLevelTarget");
			argNewLoadLevelTarget.setDirection("in");
			StateVariable stateVarLoadLevelTarget = new StateVariable();
			stateVarLoadLevelTarget.setName("LoadLevelTarget");
			argNewLoadLevelTarget.setRelatedStateVariable(stateVarLoadLevelTarget);
			// end
			actSetLoadLevelTarget.addArgument(argNewLoadLevelTarget);
			// end
			dimServ.addAction(actSetLoadLevelTarget);
			// Set up the service state table
			StateVariable sv1 = new StateVariable();
			sv1.setSendingEvents(false);
			sv1.setName("MinLevel");
			sv1.setDataType("ui1");
			StateVariable sv2 = new StateVariable();
			sv2.setSendingEvents(true);
			sv2.setName("LoadLevelStatus");
			sv2.setDataType("ui1");
			sv2.setValue("100");
			List<AllowedValue> svAllowedValueRange = new Vector<AllowedValue>();
			AllowedValue min = new AllowedValue("minimum", "0");
			AllowedValue max = new AllowedValue("maximum", "100");
			svAllowedValueRange.add(min);
			svAllowedValueRange.add(max);
			sv2.setAllowedValues(svAllowedValueRange);
			StateVariable sv3 = new StateVariable();
			sv3.setSendingEvents(false);
			sv3.setName("LoadLevelTarget");
			sv3.setDataType("ui1");
			sv3.setAllowedValues(svAllowedValueRange);
			dimServ.addStateVariable(sv1);
			dimServ.addStateVariable(sv2);
			dimServ.addStateVariable(sv3);
			// add this service to the device
			dev.addService(dimServ);
		}
		{
			// ############## SWITCH POWER SERVICE ################
			// set up the DimmingService
			Service pwrServ = new Service(this.config);
			pwrServ.setServiceType(Config.SWITCH_POWER_SERVICE_TYPE);
			// pwrServ.setServiceID("urn:schemas-upnp-org:serviceId:SwitchPower.0001");
			pwrServ.setServiceID(Config.SWITCH_POWER_SERVICE_ID);
			// set up the Actions for the Power Switch Service
			// set up the action GetStatus
			Action actGetStatus = new Action();
			actGetStatus.setName("GetStatus");
			// set up the arguments for GetStatus
			Argument argResStatus = new Argument();
			argResStatus.setName("ResultStatus");
			argResStatus.setDirection("out");
			StateVariable stateVarStatus = new StateVariable();
			stateVarStatus.setName("Status");
			argResStatus.setRelatedStateVariable(stateVarStatus);
			// end
			actGetStatus.addArgument(argResStatus);
			// end
			pwrServ.addAction(actGetStatus);
			// set up the action SetLoadLevelTarget
			Action actSetTarget = new Action();
			actSetTarget.setName("SetTarget");
			// set up the arguments for setloadleveltarget
			Argument argNewTargetValue = new Argument();
			argNewTargetValue.setName("newTargetValue");
			argNewTargetValue.setDirection("in");
			StateVariable stateVarTarget = new StateVariable();
			stateVarTarget.setName("Target");
			argNewTargetValue.setRelatedStateVariable(stateVarTarget);
			// end
			actSetTarget.addArgument(argNewTargetValue);
			// end
			pwrServ.addAction(actSetTarget);
			// set up the service state table
			StateVariable sv1 = new StateVariable();
			sv1.setSendingEvents(true);
			sv1.setName("Status");
			sv1.setDataType("boolean");
			sv1.setValue("0");
			StateVariable sv2 = new StateVariable();
			sv2.setSendingEvents(false);
			sv2.setName("Target");
			sv2.setDataType("boolean");
			pwrServ.addStateVariable(sv1);
			pwrServ.addStateVariable(sv2);
			// add this service to the device
			dev.addService(pwrServ);
		}
	}

	private void initUpnpThreads() {
		// Init one TCP-Webserver for GET Request (autmated and manual XML fetchting, etc.) [HTTP]
		this.webServer = WebServer.getWebServerForDev(this.config, this.configHttpd);
		this.webServerThread = new Thread(this.webServer);
		// Init one UDP-Webserver for the M-SEARCH and NOTIFY Requests [HTTPU]
		this.webServerUdp = WebServerUDP.getWebServerUdpForDev(this.config, this.configHttpd);
		this.webServerUdpThread = new Thread(this.webServerUdp);
		// Init one MulticastListeThread, which also listens for M-SEARCH and NOTIFY Requests [HTTPMU]
		this.multicastListen = new MulticastListenThread(this.config, this.configHttpd);
		this.multicastListenThread = new Thread(this.multicastListen);
		// Init the HeartBeatThread
		this.heartbeatThread = new Thread(new HeartbeatThread(this.config));
		// Start all the threads
		this.heartbeatThread.start();
		this.webServerThread.start();
		this.webServerUdpThread.start();
		this.multicastListenThread.start();
	}

	/**
	 * Initialize the UPnP Device
	 *
	 */

	private void initUpnpDevice() {
		this.config = new Config();
		this.configHttpd = new ConfigHttpd();
		this.initUpnpDeviceStructure();
		this.initUpnpThreads();
		this.config.getDevice().sendSSDPAlive();
		try {
			Random rand = new Random();
			int sleep = rand.nextInt(15) * 1000; // sleep for up to 15 seconds and resend
			sleep = 0;
			Thread.sleep(sleep);
		} catch (InterruptedException e) {
			logger.trace("InterruptedException occurred", e);
		}
	}

}
