/*
 * Created on 17.01.2007
 *
 */
package device.multicast;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import common.core.Config;
import common.http.ConfigHttpd;
import common.http.Header;
import common.structures.Service;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 * "The movie "Delta Force" was extremely hard to make because Chuck had to downplay his abilities. The first few cuts were completely unbelievable."
 */
public class DiscoverReplyThread implements Runnable {

	static Logger		logger			= Logger.getLogger(DiscoverReplyThread.class);

	private Config		config			= null;

	private InetAddress	senderAddress	= null;

	private int			senderPort		= 0;

	private String		scope			= "";

	private DiscoverReplyThread() {

	}

	/**
	 *
	 * @param config
	 *            Our config
	 * @param senderAddress
	 * @param sendersPort
	 * @param scope
	 *            e.g. ssdp:all or ssdp:rootdevice
	 */
	public DiscoverReplyThread(Config config, InetAddress senderAddress, int sendersPort, String scope) {
		this.config = config;
		this.senderAddress = senderAddress;
		this.senderPort = sendersPort;
		this.scope = scope;
	}

	public void run() {
		DatagramPacket send = null;
		byte[] outBuff = new byte[2 * 1024]; // allocate a 2KB buffer which should fit in almost every udp
		// implementation
		// send upnp-rootdevice
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		// setup header
		Header header = new Header(baos);
		header.addHeaderLine(ConfigHttpd.HTTP_STATUS_200);
		header.addHeaderLine("ST: upnp:rootdevice");
		header.addHeaderLine("EXT: ");
		header.addHeaderLine(Config.getHeaderServer());
		header.addHeaderLine("USN: uuid:" + config.getUuid() + "::upnp:rootdevice");
		header.addHeaderLine("CACHE-CONTROL: max-age=" + this.config.getCacheControlMaxAge());
		header.addHeaderLine("LOCATION: http://" + this.config.getLocalAddress().getHostAddress() + ":"
				+ this.config.getHttpServerPort() + "/");
		header.addHeaderLine("Content-Length: 0");
		try {
			// send header
			header.sendHeader();
			outBuff = baos.toByteArray();
			send = new DatagramPacket(outBuff, outBuff.length);
			send.setAddress(this.senderAddress);
			send.setPort(this.senderPort);
			logger.trace("Sending Reply (upnp:rootdevice) [Port: " + send.getPort() + "]: "
					+ new String(send.getData()));
			this.config.getMcSocket().send(send);
		} catch (IOException e) {
			logger.trace("Got IOException", e);
		}
		if (this.scope.trim().equalsIgnoreCase("ssdp:all")
				|| this.scope.trim().equalsIgnoreCase(this.config.getDevice().getUUID())) {
			// send uuid
			baos = new ByteArrayOutputStream();
			header = new Header(baos);
			header.addHeaderLine(ConfigHttpd.HTTP_STATUS_200);
			header.addHeaderLine("ST: uuid:" + config.getUuid());
			header.addHeaderLine("EXT: ");
			header.addHeaderLine(Config.getHeaderServer());
			header.addHeaderLine("USN: uuid:" + config.getUuid());
			header.addHeaderLine("CACHE-CONTROL: max-age=" + this.config.getCacheControlMaxAge());
			header.addHeaderLine("LOCATION: http://" + this.config.getLocalAddress().getHostAddress() + ":"
					+ this.config.getHttpServerPort() + "/");
			header.addHeaderLine("Content-Length: 0");
			try {
				header.sendHeader();
				outBuff = baos.toByteArray();
				send = new DatagramPacket(outBuff, outBuff.length);
				send.setAddress(this.senderAddress);
				send.setPort(this.senderPort);
				logger.trace("Sending Reply (uuid) [Port: " + send.getPort() + "]: " + new String(send.getData()));
				this.config.getMcSocket().send(send);
			} catch (IOException e) {
				logger.trace("IOException occurred", e);
			}
			// send uuid-schemas-device
			baos = new ByteArrayOutputStream();
			header = new Header(baos);
			header.addHeaderLine(ConfigHttpd.HTTP_STATUS_200);
			header.addHeaderLine("ST: " + this.config.getDevice().getDeviceType());
			header.addHeaderLine("EXT: ");
			header.addHeaderLine(Config.getHeaderServer());
			header.addHeaderLine("USN: uuid:" + config.getUuid() + "::urn:schemas-upnp-org:device:jLight:1");
			header.addHeaderLine("CACHE-CONTROL: max-age=" + this.config.getCacheControlMaxAge());
			header.addHeaderLine("LOCATION: http://" + this.config.getLocalAddress().getHostAddress() + ":"
					+ this.config.getHttpServerPort() + "/");
			header.addHeaderLine("Content-Length: 0");
			try {
				header.sendHeader();
				outBuff = baos.toByteArray();
				send = new DatagramPacket(outBuff, outBuff.length);
				send.setAddress(this.senderAddress);
				send.setPort(this.senderPort);
				logger.trace("Sending Reply (urn:device) [Port: \"+send.getPort()+\"]: " + new String(send.getData()));
				this.config.getMcSocket().send(send);
			} catch (IOException e) {
				logger.trace("IOException occurred", e);
			}
			// send one for every uuid-schemas-services
			ArrayList<Service> services = this.config.getDevice().getServices();
			for (int i = 0; i < services.size(); i++) {
				baos = new ByteArrayOutputStream();
				header = new Header(baos);
				header.addHeaderLine(ConfigHttpd.HTTP_STATUS_200);
				header.addHeaderLine("ST: " + services.get(i).getURN());
				header.addHeaderLine("EXT: ");
				header.addHeaderLine(Config.getHeaderServer());
				header.addHeaderLine("USN: uuid:" + config.getUuid() + "::" + services.get(i).getURN());
				header.addHeaderLine("CACHE-CONTROL: max-age=" + this.config.getCacheControlMaxAge());
				header.addHeaderLine("LOCATION: http://" + this.config.getLocalAddress().getHostAddress() + ":"
						+ this.config.getHttpServerPort() + "/");
				header.addHeaderLine("Content-Length: 0");
				try {
					header.sendHeader();
					outBuff = baos.toByteArray();
					send = new DatagramPacket(outBuff, outBuff.length);
					send.setAddress(this.senderAddress);
					send.setPort(this.senderPort);
					logger.trace("Sending Reply (urn:service) [Address: " + send.getAddress().getHostAddress()
							+ "] [Port: " + send.getPort() + "]: " + new String(send.getData()));
					this.config.getMcSocket().send(send);
				} catch (IOException e) {
					logger.trace("IOException occurred", e);
				}
			}
			this.config.getDevice().sendSSDPAlive(true, false);
		} else {
			this.config.getDevice().sendSSDPAlive(false, true);
		}
	}

}
