/*
 * Created on 18.01.2007
 *
 */
package device.multicast;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.log4j.Logger;

import common.core.Config;
import common.http.CustomHttpRequest;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * This thread will handle the NOTIFY messages to all subscribes CPs.
 *
 * @author tex
 *
 * "Once you go Norris, you are physically unable to go back."
 */
public class NotifyChangeThread implements Runnable {

	static Logger	logger		= Logger.getLogger(NotifyChangeThread.class);

	@SuppressWarnings("unused")
	private Config	config		= null;

	private String	callback	= "";

	private String	sid			= "";

	private String	value		= "";

	private String	varName		= "";

	private int		seq			= 0;

	/**
	 *
	 * @param config
	 *            Our Config
	 * @param callback
	 *            The Callback URL which gets called
	 * @param sid
	 *            the SID of this subscription
	 * @param varName
	 *            the name of the variable that has changed
	 * @param value
	 *            the new value
	 * @param seq
	 *            the SEQ value
	 */
	public NotifyChangeThread(Config config, String callback, String sid, String varName, String value, int seq) {
		this.config = config;
		this.callback = callback;
		this.sid = sid;
		this.value = value;
		this.varName = varName;
		this.seq = seq;
	}

	public void run() {
		try {
			logger.trace("Sending notify to " + this.callback);
			// generate notifyrequest
			CustomHttpRequest req = new CustomHttpRequest(new URL(this.callback), "NOTIFY");
			req.addHeaderLine("CONTENT-TYPE: text/xml");
			req.addHeaderLine("NT: upnp:event");
			req.addHeaderLine("NTS: upnp:propchange");
			req.addHeaderLine("SID: " + this.sid);
			req.addHeaderLine("SEQ: " + this.seq);
			String payload = "<e:propertyset xmlns:e=\"urn:schemas-upnp-org:event-1-0\">\r\n" + "  <e:property>\r\n"
					+ "    <" + this.varName + ">" + this.value + "</" + this.varName + ">\r\n" + "  </e:property>\r\n"
					+ "</e:propertyset>";
			req.setPayload(payload);
			logger.trace("Sending notify to " + this.callback + ". Data:\n" + req.toString());
			// wait between 300 and 1000 ms
			Config.waitRandom(this.config, 30, 100);
			// send request
			req.sendRequest();
		} catch (MalformedURLException e) {
			logger.trace("MalformedURLExcpetion occurred", e);
		} catch (IOException e) {
			logger.trace("IOException occurred", e);
		}
	}

}
