/*
 * Created on 12.12.2006
 *
 */
package device.multicast;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;

import org.apache.log4j.Logger;

import common.core.Config;
import common.http.ConfigHttpd;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 * "Ninjas want to grow up to be just like Chuck Norris. But usually they grow up just to be killed by Chuck Norris."
 */
public class SSDPAliveThread implements Runnable {

	static Logger			logger	= Logger.getLogger(SSDPAliveThread.class);

	private Config			config	= null;

	private String			nt		= "";

	private String			usn		= "";

	private MulticastSocket	ms		= null;

	/**
	 *
	 * @param conf
	 *            Our Config
	 * @param nt
	 *            The NT Header
	 * @param usn
	 *            The USN Header
	 */
	public SSDPAliveThread(Config conf, String nt, String usn) {
		this.config = conf;
		this.nt = nt;
		this.usn = usn;
		try {
			this.ms = new MulticastSocket();
			this.ms.joinGroup(InetAddress.getByName(Config.MULTICAST_GROUP));
			logger.trace("Constructor() - Bound MulticastSocket to Port: " + this.ms.getPort());
		} catch (IOException e) {
			logger.error(e);
		}
	}

	public void run() {
		// generate Multicast notify
		String aliveMessage = "";
		aliveMessage = "NOTIFY * HTTP/" + ConfigHttpd.HTTP_VERSION + Config.CRLF;
		aliveMessage += "LOCATION: http://" + this.config.getLocalAddress().getHostAddress() + ":"
				+ this.config.getHttpServerPort() + "/" + Config.CRLF;
		aliveMessage += "HOST: 239.255.255.250:1900" + Config.CRLF;
		aliveMessage += Config.getHeaderServer() + Config.CRLF;
		aliveMessage += "NTS: ssdp:alive" + Config.CRLF;
		aliveMessage += "USN: " + this.usn + Config.CRLF;
		aliveMessage += "CACHE-CONTROL: max-age=" + this.config.getCacheControlMaxAge() + Config.CRLF;
		aliveMessage += "NT: " + this.nt + Config.CRLF;
		aliveMessage += "Content-Length: 0" + Config.CRLF;
		aliveMessage += Config.CRLF;
		DatagramPacket alivePacket = new DatagramPacket(aliveMessage.getBytes(), aliveMessage.getBytes().length,
				this.config.getGroup(), Config.MULTICAST_PORT);

		logger.trace("run() - Sending: " + new String(alivePacket.getData()));

		try {
			this.ms.send(alivePacket);
		} catch (IOException e) {
			logger.error(e);
		}
	}

}
