/*
 * Created on 12.12.2006
 *
 */
package device.multicast;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.MulticastSocket;

import org.apache.log4j.Logger;

import common.core.Config;
import common.http.ConfigHttpd;

/**
 * <pre>
 *          Copyright (c) 2007 Dominik Schulz
 *          Copyright (c) 2007 Florian Lindner
 *          Copyright (c) 2007 Philip Hartmann
 *
 *          This file is part of jUPnP.
 *
 *          jUPnP is free software; you can redistribute it and/or modify
 *          it under the terms of the GNU General Public License as published by
 *          the Free Software Foundation; either version 2 of the License, or
 *          (at your option) any later version.
 *
 *          jUPnP is distributed in the hope that it will be useful,
 *          but WITHOUT ANY WARRANTY; without even the implied warranty of
 *          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *          GNU General Public License for more details.
 *
 *          You should have received a copy of the GNU General Public License
 *          along with jUPnP; if not, write to the Free Software
 *          Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * </pre>
 *
 * @author tex
 *
 * "Chuck Norris can set ants on fire with a magnifying glass. At night."
 */
public class SSDPByeByeThread implements Runnable {

	static Logger	logger	= Logger.getLogger(SSDPByeByeThread.class);

	private Config	config	= null;

	private String	nt		= "";

	private String	usn		= "";

	/**
	 *
	 * @param conf
	 *            Our Config
	 * @param nt
	 *            The NT Header
	 * @param usn
	 *            The USN Header
	 */
	public SSDPByeByeThread(Config conf, String nt, String usn) {
		this.config = conf;
		this.nt = nt;
		this.usn = usn;
	}

	/**
	 * ByeBye Thread run method
	 */
	public void run() {
		// generate multicast bye bye
		String byeByeMessage = "";
		byeByeMessage = "NOTIFY * HTTP/" + ConfigHttpd.HTTP_VERSION + Config.CRLF;
		byeByeMessage += "LOCATION: http://" + this.config.getLocalAddress().getHostAddress() + ":"
				+ this.config.getHttpServerPort() + "/" + Config.CRLF;
		byeByeMessage += "HOST: 239.255.255.250:1900" + Config.CRLF;
		byeByeMessage += Config.getHeaderServer() + Config.CRLF;
		byeByeMessage += "NTS: ssdp:byebye" + Config.CRLF;
		byeByeMessage += "USN: " + this.usn + Config.CRLF;
		byeByeMessage += "CACHE-CONTROL: max-age=" + this.config.getCacheControlMaxAge() + Config.CRLF;
		byeByeMessage += "NT: " + this.nt + Config.CRLF;
		byeByeMessage += "Content-Length: 0" + Config.CRLF;
		byeByeMessage += Config.CRLF;

		DatagramPacket byeByePacket = new DatagramPacket(byeByeMessage.getBytes(), byeByeMessage.getBytes().length,
				this.config.getGroup(), Config.MULTICAST_PORT);

		MulticastSocket ms = this.config.getMcSocket();

		logger.trace("run() - Sending: " + byeByeMessage);

		try {
			ms.send(byeByePacket);
		} catch (IOException e) {
			logger.error(e);
		}
	}

}
